/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996-1999 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <pwd.h>
#include <utmp.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
#include "log.h"
#include "sm.h"
#include "clobber_user.h"
#include "notify.h"
#include "get_if_stats.h"
#include "monitor.h"
#include "net_server.h"

char our_hostname[STR_LEN + 1];

/***************************************************************************
*
***************************************************************************/
void kill_daemon(int sig)
{
  running = 0;
}

void init(void)
{
  register int x;

  for(x = 0;x < MAX_KONS;x++)
    kon[x].s = -1;
  us_hr_clr(&us_hr);
  signal(SIGINT,kill_daemon);
  signal(SIGKILL,kill_daemon);
  signal(SIGTERM,kill_daemon);
  signal(SIGHUP,SIG_IGN);
  signal(SIGPIPE,SIG_IGN);
  old_save_time = old_timer = timer = time(NULL);
}

/***************************************************************************
*
***************************************************************************/
void daemonise(void)
{
  void furk(void)
  {
    switch(fork())
    {
      case -1: //error
        perror("daemonise() fork()");
        exit(0);
      case 0: //child
        break;
      default:
        exit(0);
    }
  }

  if(daemon_mode)
  {
    register int x;

    fflush(NULL);
    furk();
    for(x = 0;x <= getdtablesize();++x)
      close(x);
    setsid();
    furk();
    umask(0);
    chdir("/");
    x = open("/dev/null",O_RDWR);
    dup(x);
    dup(x);
  }
}

/***************************************************************************
*
***************************************************************************/
void main_loop(void)
{
  while(running)
  {
    old_timer = timer;
    timer = time(NULL);
    time_delta = timer - old_timer;

    scan_utmp();
    monitor();
    LOG_NMALLOC();
    while(running)
    {
      register int rt_loop_time;

      rt_loop_time = time(NULL);
      check_new_kons();
      service_kons();
      if((rt_loop_time - old_save_time) > us_hr.save_time)
      {
        old_save_time = rt_loop_time;
        save_data_file();
        log_msg('I',"Saved datafile (scheduled save)",0);
      }
      if((rt_loop_time - timer) >= us_hr.sleep_time)
        break;
      nuke_zombies();

      if(!no_kons)
        sleep(1);
      else
        usleep(20000);
    }
  }
  log_msg('I',"Saving datafile before exit",1);
  save_data_file();
}

/***************************************************************************
*
***************************************************************************/
int main(int argc,char *argv[])
{
  register int x;
  char str[STR_LEN];
  struct passwd *pw;

  argv_ptr = argv;
  printf(WELCOME);
  for(x = 0;x < argc;x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          printf(
            DISCLAIMER
            "\nUsage: clobberd <options>\n"
             " -h        Display this help info\n"
             " -!        Run as normal process, logs sent to screen.\n"
             " -t        Test mode. For non-intrusive testing.\n"
             "\n"
          );
          return 0;
        case 't':
          test_mode = 1;
          break;
        case '!':
          daemon_mode = 0;
          break;
      }
    }
  }
  pw = getpwuid(getuid());
  if(pw != NULL)
  {
    if(strcmp(pw->pw_name,"root"))
    {
      printf("Must be root to run clobberd. Aborting process.\n");
      return -1;
    }
  }
  sprintf(str,"(%s)",argv[0]);
  if(find_process(str,0) > 0)
  {
    printf("You are already running clobberd. Aborting this process..\a\n");
    return -1;
  }
  init();
  daemonise();
  if(read_boot_conf())
  {
    if(gethostname(our_hostname,STR_LEN) != -1)
    {
      listen_s = server_bind_socket(PORT);
      if(listen_s)
      {
        if_sock = socket(AF_INET,SOCK_STREAM,0);
        if(if_sock != -1)
        {
          log_msg('I',daemon_started_mess,1);
#ifdef CLOBBER_LOGIN_PROCESSES_ONLY
          log_msg('I',"Will clobber login processes only",1);
#endif
#ifdef PROBE_MALLOC
          log_msg('I',">>>> NMALLOC (debug) logging on <<<<",1);
#endif
          if(test_mode)
            log_msg('I',"]]]]]] TEST MODE (won't clobber users) [[[[[[[",1);
          sprintf(str,"Running on %s:%d and priming..",our_hostname,PORT);
          log_msg('I',str,1);
          log_msg('I',"Loading datafile.",1); 
          load_data_file();
          main_loop();
          free_us();
          free_um();
          kill_all_kons();
          log_msg('I',daemon_ended_mess,1);
          close(if_sock);
        }
        else
          log_error("Couldn't creat PPP socket");
      }
      else
      {
        sprintf(str,"Couldn't bind Port %d",PORT);
        log_error(str);
      }
    }
    else
      log_error("Couldn't resolve current hostname");
  }
  else
    log_error("Error reading boot file (read_boot_conf())");
  PRINT_NMEM();
  return 0;
}
