/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 96,97,98,1999 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
****************************************************************************
****************************************************************************
***************************************************************************/
#define VERSION "4.10"
#define DATED    __DATE__
#define WELCOME \
"Clobberd version " VERSION " (" DATED "), Copyright (C) 96,97,98,1999 Jason Nunn.\n\n"
#define DISCLAIMER \
"Clobberd is under the GNU GENERAL PUBLIC LICENSE AGREEMENT, and\n" \
"comes with ABSOLUTELY NO WARRANTY. This is free software, and you are\n" \
"welcome to redistribute it under certain conditions.\n\n" \
"   Contact: 32 Rothdale Rd, Moil Darwin NT 0810, AUSTRALIA\n" \
"     Email: jsno@downunder.net.au\n" \
"       WWW: www.downunder.net.au/~jsno\n" \
"\n"

/*this is 64 bit unsigned on glib 2.1.*/
typedef unsigned long long int _UD_64;

#define STR_LEN             256
#define STR_GEN_LEN         40
#define STR_LEN_LN          80
#define STR_LEN_INT         8
#define STR_LEN_REAL        12
#define STR_USER_LEN        32
#define STR_BIG             2048
#define STR_TTY_LEN         12
#define STR_SMALL           16

#define BOOT_FILE           VAR_PATH "/boot.conf"
#define DATA_FILE           VAR_PATH "/clobberd.db"
#define LOG_FILE            VAR_PATH "/log"
#define REPORT_FILE         VAR_PATH "/report"

#define CLOB_SIGNATURE      VAR_PATH "/signature.txt"

#define CLOB_INDEX          0
#define CLOB_MESS           VAR_PATH "/m_banned.txt"
#define CLOB_MULTIPLE_INDEX 1
#define CLOB_MULTPLE_MESS   VAR_PATH "/m_multiple.txt"
#define ACCT_EXPIRED_INDEX  2
#define ACCT_EXPIRED_MESS   VAR_PATH "/m_expired.txt"
#define EXCEEDED_TT_INDEX   3
#define EXCEEDED_TT_MESS    VAR_PATH "/m_total_time.txt"
#define EXCEEDED_DT_INDEX   4
#define EXCEEDED_DT_MESS    VAR_PATH "/m_daily_time.txt"
#define EXCEEDED_TNET_INDEX 5
#define EXCEEDED_TNET_MESS  VAR_PATH "/m_total_net.txt"
#define EXCEEDED_DNET_INDEX 6
#define EXCEEDED_DNET_MESS  VAR_PATH "/m_daily_net.txt"
#define UACT_TIMEOUT_INDEX  7
#define UACT_TIMEOUT_MESS   VAR_PATH "/m_activity_to.txt"

#define RECV_BUF_LEN        512

#if PROBE_MALLOC
  #define NMALLOC     nmalloc
  #define NREALLOC    nrealloc
  #define NFREE       nfree
  #define PRINT_NMEM  print_nmem
  #define LOG_NMALLOC log_nmalloc
#else
  #define NMALLOC     malloc
  #define NREALLOC    realloc
  #define NFREE       free
  #define PRINT_NMEM  print_nmem_nothing
  #define LOG_NMALLOC print_nmem_nothing
#endif

/*************************************************/
typedef struct
{
  char *mod;
  char *repl;
} tmlist;

/*************************************************/
#define TTY_T_SIZE       256
#define EXEMPT_USER_SIZE 32

typedef struct
{
  char passwd[STR_GEN_LEN + 1];
  int  mfd_tty_t_size;
  char mfd_tty_t[TTY_T_SIZE][STR_TTY_LEN + 1];
  int  exempt_user_size;
  char exempt_user[EXEMPT_USER_SIZE][STR_USER_LEN + 1];
} tboot_st;

/*************************************************/
#define DT_LIM_SIZE                 8

#define TUSF_NOTIFY_VIA_EMAIL       0x00001
#define TUSF_EXPT_TOTAL_TIME        0x00002
#define TUSF_EXPT_DAILY_TIME        0x00004
#define TUSF_EXPT_EXPIRY_DATE       0x00008
#define TUSF_EXPT_NICE              0x00010
#define TUSF_EXPT_TOTAL_UL_LIM      0x00020
#define TUSF_EXPT_TOTAL_DL_LIM      0x00040
#define TUSF_EXPT_DAILY_UL_LIM      0x00080
#define TUSF_EXPT_DAILY_DL_LIM      0x00100
#define TUSF_INSTANT_PURGE_DT       0x00200
#define TUSF_CLOBBER_MULT_LOGINS    0x00400
#define TUSF_CLOBBER                0x00800
#define TUSF_EXPT_ACT_TIMEOUT       0x01000

typedef struct
{
  char d_fr;            // 0 = Mon, 6 = Sun
  char d_to;
  char t_fr;            // 1am - 12pm
  char t_to;
  int  limit;
} tdt_lim;

typedef struct
{
  char day;
  char month;
  int  year;
} tdate_st;

typedef struct _tus_ll
{
  int            flags;
  char           user[STR_USER_LEN + 1];
  char           email[STR_LEN_LN + 1];
  int            tt_lim;
  tdt_lim        dt_lim[DT_LIM_SIZE];
  tdate_st       expiry_date;
  int            nice_factor;
  int            t_ul_lim;    /*Mb*/
  int            t_dl_lim;    /*Mb*/
  int            d_ul_lim;    /*Mb*/
  int            d_dl_lim;    /*Mb*/
  int            act_timeout;
  struct _tus_ll *next;
} tus_ll;

#define TUS_LL_SIZE (sizeof(tus_ll) - sizeof(tus_ll *))

typedef struct _tus_btree
{
  tus_ll            *entry;
  struct _tus_btree *left;
  struct _tus_btree *right;
} tus_btree;

#define TUSF_HR_G_RUN_LEVEL(n)         (n & 3)
#define TUSF_HR_MIN_FREE_DIALIN_EXEMPT 0x10
#define TUSF_HR_DEFAULT_US_ENABLED     0x20

#define TUSF_HR_S_RUN_LEVEL(v,n)       v = ((v & ~3) | n)

typedef struct
{
  int    flags;
  char   root_emailname[STR_USER_LEN + 1];
  char   email_host[STR_GEN_LEN + 1];
  int    sleep_time;
  int    grace_time;
  int    grace_volume;
  int    save_time;
  int    min_free_dialins;
  tus_ll default_us;
} tus_hr;

/*************************************************/
#define TINFO_TYPE_UNKNOWN     0
#define TINFO_TYPE_LOGIN       1
#define TINFO_TYPE_PPP_LOGIN   2
#define TINFO_TYPE_PPP_PAP     3
#define TINFO_TYPE_PPP_UNKNOWN 4

typedef struct _ttinfo_ll
{
  int               type;
  int               pid;                  /*pid & term id together*/
  char              tty[STR_TTY_LEN + 1]; /*determine unique session*/
  char              nif[STR_SMALL + 1]; /*network interface*/
  int               ul;
  int               dl;
  int               old_ul;
  int               old_dl;
  int               last_active;
  struct _ttinfo_ll *next;
} ttinfo_ll;

#define TTINFO_LL_SIZE (sizeof(ttinfo_ll) - sizeof(ttinfo_ll *))

/*for caching network interface associations*/
typedef struct _tnif_ll
{
  int             pid;
  char            nif[STR_SMALL + 1];
  struct _tnif_ll *next;
} tnif_ll;

/*************************************************/
#define TUMF_TT_GRACED  0x01
#define TUMF_DT_GRACED  0x02
#define TUMF_TUL_GRACED 0x04
#define TUMF_TUD_GRACED 0x08
#define TUMF_DUL_GRACED 0x10
#define TUMF_DUD_GRACED 0x20

typedef struct _tum_ll
{
/*stored fields*/
  char           user[STR_USER_LEN + 1];
  int            no_active;
  int            tt_used;
  int            dt_used;
  _UD_64         t_ul; /*stored in bytes*/
  _UD_64         t_dl; /*64 bit integer*/
  _UD_64         d_ul;
  _UD_64         d_dl;

/*runtime (temp) fields*/
  int            last_active;
  int            dt_zone_limit;
  int            dt_set_limit;
  _UD_64         old_ul;
  _UD_64         old_dl;
  int            tmp_flags;
  ttinfo_ll      *tinfo_ll;
  struct _tum_ll *next;
} tum_ll;

#define TUM_LL_SIZE (sizeof(tum_ll) - sizeof(tum_ll *))

/*************************************************/
#define TKON_AUTHORISED   0x1
#define TKON_US_HR_LOCKED 0x2

#define TKON_MODE_CMD       0
#define TKON_MODE_S_US_HR   1
#define TKON_MODE_S_US      2
#define TKON_MODE_S_UM      3

typedef struct
{
  int    flags;
  int    mode;
  int    s;
  char   hostname[STR_GEN_LEN + 1];
  int    port;
  int    time;
  char   user_lock[STR_USER_LEN + 1];
} tkon;
