/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 96,97,98,1999 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* get network volume stats from network interfaces
*
* this section gets bytes send/recv'ed from network interfaces they are 
* using and updates their network volumes in um struct
*
* nb/ at the request of some users, i'm slowly begining to comment the
* code.
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <utmp.h>
#include <pwd.h>
#include <dirent.h>
#include <net/if_ppp.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
#include "log.h"
#include "sm.h"
#include "monitor.h"

int if_sock;

/***************************************************************************
* here we cache the ppp file locks in a linked list for semi-fast lookups.
***************************************************************************/
int test_ppplockfile(char *str)
{
  if((str[0] == 'p') &&
     (str[1] == 'p') &&
     (str[2] == 'p') &&
     (strstr(str,".pid") != NULL))
    return 1;
  return 0;
}

void cache_nif(void)
{
  DIR *dp;

  if((dp = opendir(PPPD_RUN_PID_DIR)) != NULL)
  {
    struct dirent *dir_entry;

    for(;;)
    {
      dir_entry = readdir(dp);
      if(dir_entry == NULL)
        break;
      if(test_ppplockfile(dir_entry->d_name))
      {
        int i = 0;
        char ppp_nif[STR_SMALL + 1] = "";
        FILE *fp;
        char str[STR_LEN];

        for(;;)
        {
          if((dir_entry->d_name[i] == 0) || (dir_entry->d_name[i] == '.'))
            break;
          ppp_nif[i] = dir_entry->d_name[i];
          i++;
        }
        ppp_nif[i] = 0;

        /*get the pid number inside lock file*/
        strcpy(str,PPPD_RUN_PID_DIR);
        strcat(str,"/");
        strcat(str,dir_entry->d_name);
        if((fp = fopen(str,"r")) != NULL)
        {
          fscanf(fp,"%s",str);
          fclose(fp);
          append_nif(ppp_nif,atoi(str));
        }
      }
    }
    closedir(dp);
  }
}

/***************************************************************************
* gets ppp stats for a given tinfo (terminal)
*
* Nb/ sensing and recording ppp stats is now becoming a beasty exercise..
*
* these routines are designed to sense pppd 2.3.x using proc fs 2.2.x. i
* don't know how portable it is cross pppd's..
*
* don't have to worry about locking coz we're not touching the volume's in
* um struct. that will be done later in monitor()
***************************************************************************/
void get_ppp_stats(ttinfo_ll *tinfo,struct utmp *utmp_ptr)
{
  char str[STR_LEN + 1];
  struct ifreq ifr;
  struct ppp_stats stats;
  tnif_ll *p;

  /*
   * first we'll look for a PAP connection. it's fairly arcane. pppd (2.3.x)
   * doesn't exactly jump out and tell you whose if it is, and what type
   * it is.. however, it leaves clues..
   */
  p = find_nif(utmp_ptr->ut_pid);
  if(p != NULL)
  {
    /*
     * hmmm. ppp pid found in utmp struct. looks like PAP..
     */
    tinfo->type = TINFO_TYPE_PPP_PAP;
    strcpy(tinfo->nif,p->nif);
  }
  else
  {
    /*
     * hmmm.. it's not PAP. lets check for bogg standard login PPP
     */
    p = nif_ll;
    while(p != NULL)
    {
      FILE *fp;

      /*
       * look at the old uid value for the pppd process and compare
       * it to the user's in the utmp entry.
       */
      sprintf(str,"/proc/%d/status",p->pid);
      if((fp = fopen(str,"r")) != NULL)
      {
        for(;;)
        {
          if(fgets(str,STR_LEN,fp) == NULL)
            break;
          if(!memcmp(str,"Uid:",4))
          {
            struct passwd *pw;
            int p_uid = -1;

            sscanf(str,"%*s %d",&p_uid);
            pw = getpwuid(p_uid);
            if(pw != NULL)
            {
              if(!strcmp(pw->pw_name,utmp_ptr->ut_user))
              {
                /*
                 * login PPP connection found.
                 */
                tinfo->type = TINFO_TYPE_PPP_LOGIN;
                strcpy(tinfo->nif,p->nif);
              }
            }
            break;
          }
        }
        fclose(fp);
      }

      if(tinfo->type == TINFO_TYPE_PPP_LOGIN)
        break;                             /*found?, exit login pppd search*/

      p = p->next;
    }
  }

  if(tinfo->type == TINFO_TYPE_UNKNOWN)  /*no ppp if found, exit*/
    return;

  /*
   * now we have a ppp-if for a user, we can now suck some volume
   * stats out of it.
   */
  memset(&ifr,0,sizeof(struct ifreq));
  ifr.ifr_ifru.ifru_data = (caddr_t)&stats;
  strcpy(ifr.ifr_ifrn.ifrn_name,tinfo->nif);
  if(ioctl(if_sock,SIOCGPPPSTATS,&ifr) != -1)
  {
    int d_ul,d_dl;

    if((tinfo->old_ul == -1) && (tinfo->old_dl == -1))
    {

      /*
       * if it's a brand new terminal entry, and this is the first
       * time, then set delta's to 0. we do this because if clobberd
       * is currently monitoring a bunch of ppp terminals, then aborts
       * suddenly, and is then restarted back up, if the old_* settings
       * are set to 0, then delta calculations will be data they have
       * already been volumed for. therefore, they'll get a doubling up of
       * volume recordings.
       */
      d_ul = 0;
      d_dl = 0;
    }
    else
    {
      d_ul = stats.p.ppp_ibytes - tinfo->old_ul;
      d_dl = stats.p.ppp_obytes - tinfo->old_dl;
    }
    tinfo->old_ul = stats.p.ppp_ibytes;
    tinfo->old_dl = stats.p.ppp_obytes;
    (tinfo->ul) += d_ul;
    (tinfo->dl) += d_dl;

    /*
     * if data has transferred since last wakeup, then user is therefore
     * active
     */
    if(d_ul || d_dl)
      tinfo->last_active = timer;
  }
}
