/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "defs.h"
#include "log.h"

char     *addr = "localhost";
tboot_st boot_st;
char     **argv_ptr;

/*universal structures*/
tnif_ll   *nif_ll = NULL,**nif_ll_tail = &nif_ll;
tus_hr    us_hr;
tus_ll    *us_ll = NULL;
tus_btree *us_btree = NULL;
tum_ll    *um_ll = NULL;

char  *notify_label = "Notify";
char  *warning_label = "Warning";
char  *error_label = "Error";
char  *yes_label = "Yes";
char  *no_label  = "No";
char  *unknown_label = "UNKNOWN";
char  *exempt_label = "Exempt";
char  *na_label = "n/a";
char  *verbose_label = "Verbose";
char  *normal_label = "Normal";
char  *none_label = "None";
char  *daemon_started_mess = "--Daemon Started----";
char  *daemon_ended_mess = "--Daemon ended------";
char  *dt_table[] = {"Mon","Tue","Wed","Thu","Fri","Sat","Sun"};
char  *tm_table[] = {"00","01","02","03","04","05",
                     "06","07","08","09","10","11",
                     "12","13","14","15","16","17",
                     "18","19","20","21","22","23"};
char  *term_type[] =
{
  "Unknown",
  "Shell Login",
  "Pppd Login",
  "Pppd PAP",
  "PPP Unknown"
};

/*cmd codes*/
char *cmd_ok     = ";ok";
char *cmd_er     = ";er";

char *cmd_quit   = ";quit";
char *cmd_pass   = ";pass";
char *cmd_stat   = ";stat";

char *cmd_gushr  = ";gushr";
char *cmd_sushr  = ";sushr";
char *cmd_lushr  = ";lushr";
char *cmd_uushr  = ";uushr";

char *cmd_gusu   = ";gusu";
char *cmd_gus    = ";gus";
char *cmd_gus2   = ";gus2";
char *cmd_sus    = ";sus";
char *cmd_nus    = ";nus";
char *cmd_dus    = ";dus";

char *cmd_gumu   = ";gumu";
char *cmd_gum    = ";gum";
char *cmd_sum    = ";sum";
char *cmd_dum    = ";dum";

char *cmd_luser  = ";luser";
char *cmd_uuser  = ";uuser";

char *cmd_stime  = ";stime";

char *cmd_tpurg  = ";tpurg";
char *cmd_dpurg  = ";dpurg";

char *cmd_trept  = ";trept";

char *cmd_ustat  = ";ustat";

char *cmd_die    = ";die";

char *cmd_save   = ";save";

char *cmd_gtinf  = ";gtinf";

/*struct tokens*/
#define CMD_DT_SIZE 7

/*us hr*/
char *dtushr_flags            = ":hFlags";
char *dtushr_root_emailname   = ":hRUser";
char *dtushr_email_host       = ":hEHost";
char *dtushr_sleep_time       = ":hSleep";
char *dtushr_grace_time       = ":hGTime";
char *dtushr_grace_volume     = ":hGVolm";
char *dtushr_save_time        = ":hSaveT";
char *dtushr_min_free_dialins = ":hMinDn";

/*us ll*/
char *dtusll_flags            = ":sFlags";
char *dtusll_user             = ":sUserN";
char *dtusll_email            = ":sEmail";
char *dtusll_tt_lim           = ":sTtLim";
char *dtusll_dt_lim           = ":sDtLim";
char *dtusll_expiry_date      = ":sEDate";
char *dtusll_nice_factor      = ":sNiceF";
char *dtusll_t_ul_lim         = ":sT_ULL";
char *dtusll_t_dl_lim         = ":sT_DLL";
char *dtusll_d_ul_lim         = ":sD_ULL";
char *dtusll_d_dl_lim         = ":sD_DLL";
char *dtusll_act_timeout      = ":sACTTO";

/*um ll*/
char *dtumll_user             = ":mUserN";
char *dtumll_no_active        = ":mNActv";
char *dtumll_grace_countdown  = ":mGr_CD";
char *dtumll_last_active      = ":mLastA";
char *dtumll_tt_used          = ":mTTusd";
char *dtumll_dt_zone_limit    = ":mDTZon";
char *dtumll_dt_set_limit     = ":mDTslm";
char *dtumll_dt_used          = ":mDTusd";
char *dtumll_t_ul             = ":mT_ULs";
char *dtumll_t_dl             = ":mT_DLs";
char *dtumll_d_ul             = ":mD_ULs";
char *dtumll_d_dl             = ":mD_DLs";
char *dtumll_tmp_flags        = ":mTflag";

/*tinfo ll*/
char *dttinfo_ll_type         = ":tTtype";
char *dttinfo_ll_pid          = ":t__PID";
char *dttinfo_ll_tty          = ":t__TTY";
char *dttinfo_ll_nif          = ":tNetIf";
char *dttinfo_ll_ul           = ":tNetUL";
char *dttinfo_ll_dl           = ":tNetDL";
char *dttinfo_ll_last_active  = ":tLastA";

/***************************************************************************
*
***************************************************************************/
void client_init(void)
{
  signal(SIGPIPE,SIG_IGN);
  signal(SIGINT,SIG_IGN);
}

/***************************************************************************
*
***************************************************************************/
int freadln(FILE *fp,char *str)
{
  str[0] = 0;
  fgets(str,STR_LEN,fp);
  if((str[0] != '#') && (str[0] >= 32)) return 1;
  return 0;
}

int read_boot_conf(void)
{
  FILE *fp;
  char str[STR_LEN + 1],substr[STR_LEN + 1];

  int get_par(char *d_str,char *s_str,char *key,int max)
  {
    register int siz = strlen(key);
    if(!strncmp(s_str,key,strlen(key)))
      if(strlen(s_str) > siz)
      {
        char *ptr;

        strncpy(d_str,s_str + siz + 1,max);
        d_str[max] = 0;
        if((ptr = memchr(d_str,'\n',max)) != NULL)
          *ptr = 0;
        return 1;
      }
    return 0;
  }

  if((fp = fopen(BOOT_FILE,"r")) == NULL)
    return 0;
  boot_st.mfd_tty_t_size = 0;
  boot_st.exempt_user_size = 0;
  while(!feof(fp))
    if(freadln(fp,str))
    {
      get_par(boot_st.passwd,str,"passwd:",STR_GEN_LEN);
      if(get_par(substr,str,"add_tty:",STR_GEN_LEN))
      {
        if(boot_st.mfd_tty_t_size < TTY_T_SIZE)
        {
          strncpy(boot_st.mfd_tty_t[boot_st.mfd_tty_t_size],
            substr,STR_TTY_LEN);
          boot_st.mfd_tty_t_size++;
        }
      }
      if(get_par(substr,str,"add_exempt_user:",STR_GEN_LEN))
      {
        if(boot_st.exempt_user_size < EXEMPT_USER_SIZE)
        {
          strncpy(boot_st.exempt_user[boot_st.exempt_user_size],
            substr,STR_USER_LEN);
          boot_st.exempt_user_size++;
        }
      }
    }
  fclose(fp);
  return 1;
}

/***************************************************************************
*
***************************************************************************/
void str_mm_ss(int t,char *s)
{
  char ms[4];

  if(t < 0) t = 0;
  sprintf(ms,"%d",(int)(t % 60));
  if(strlen(ms) == 1)
  {
    ms[1] = ms[0];
    ms[0] = '0';
    ms[2] = 0;
  }
  sprintf(s,"%d:%s",(int)(t / 60),ms);
}

void str_hh_mm(int t,char *s)
{
  char ms[4];

  if(t < 0) t = 0;
  if((t % 60) < 30)
    t /= 60;
  else
  {
    t /= 60;
    t++;
  }
  sprintf(ms,"%d",(int)(t % 60));
  if(strlen(ms) == 1)
  {
    ms[1] = ms[0];
    ms[0] = '0';
    ms[2] = 0;
  }
  sprintf(s,"%d:%s",(int)(t / 60),ms);
}

void range_check(int *val,int lim1,int lim2)
{
  if(*val < lim1) *val = lim1;
  if(*val > lim2) *val = lim2;
}

void compose_date(tdate_st *d,char *str)
{
  register int x;

  sprintf(str,"%2d:%2d:%4d",d->day,d->month,d->year);
  for(x = 0;x < 10;x++)
    if(str[x] == ' ') str[x] = '0';
}

double rnd2meg(int val)
{
  double d;
  register int m;
  
  d = val / 0x100000;  
  m = val % 0x100000;
  d += ((double)m / 0x100000);
  return d;
}

double rnd2meg64(_UD_64 val)
{
  double d;
  register int m;

  d = val / 0x100000;
  m = val % 0x100000;
  d += ((double)m / 0x100000);
  return d;
}

/***************************************************************************
*
***************************************************************************/
int server_bind_socket(int port)
{
  register int s;
  struct sockaddr_in myaddr_in;
  int reuse_opt = 1;

  memset((char *)&myaddr_in,0,sizeof(struct sockaddr_in));
  myaddr_in.sin_family = AF_INET;
  myaddr_in.sin_addr.s_addr = INADDR_ANY;
  myaddr_in.sin_port = htons(port);
  if((s = socket(AF_INET,SOCK_STREAM,0)) != -1)
  {
    setsockopt(s,SOL_SOCKET,SO_REUSEADDR,&reuse_opt,sizeof(reuse_opt));
    if(bind(s,(struct sockaddr *)&myaddr_in,
      sizeof(struct sockaddr_in)) != -1)
      if(listen(s,5) != -1)
        return s;
  }
  return 0;
}

int client_bind_socket(char *addr,int port)
{
  register int s;
  struct hostent *hp;
  struct sockaddr_in saddr_in;

  memset((char *)&saddr_in,0,sizeof(struct sockaddr_in));
  if((hp = gethostbyname(addr)) != NULL)
  {
    saddr_in.sin_family = AF_INET;
    saddr_in.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
    saddr_in.sin_port = htons(port);
    if((s = socket(AF_INET,SOCK_STREAM,0)) != -1)
      if(connect(s,(struct sockaddr *)&saddr_in,
        sizeof(struct sockaddr_in)) != -1)
        return s;
  }
  return 0;
}

void close_socket(int s)
{
  shutdown(s,2);
  close(s);
}

/***************************************************************************
*
***************************************************************************/
int find_process(char *process_name,int mode)
{
  DIR *dp;
  struct dirent *dir_entry;
  int proc_id = -1;

  if((dp = opendir("/proc")) != NULL)
  {
    while((dir_entry = readdir(dp)) != NULL)
    {
      int e_pid;

      if(sscanf(dir_entry->d_name,"%d",&e_pid))
      {
        FILE *fp;
        char str[STR_LEN];

        strcpy(str,"/proc/");
        strcat(str,dir_entry->d_name);
        strcat(str,"/stat");
        if((fp = fopen(str,"r")) != NULL)
        {
          char substr[STR_LEN];

          fgets(str,STR_LEN,fp);
          fclose(fp);
          if(sscanf(str,"%*u %s",substr))
          {
            if(!strcmp(substr,process_name))
            {
              if(mode)
                proc_id = e_pid;
              else
                proc_id++;
            }
          }
        }
      }
    }
    closedir(dp);
  }
  return proc_id;
}

/***************************************************************************
****************************************************************************
* universal packet coder/decoder code
****************************************************************************
***************************************************************************/

/***************************************************************************
*
***************************************************************************/
void decomp_str(char *dest,char *src)
{
  if(sscanf(src,"%*s %s",dest) == -1)
    dest[0] = 0;
}

/***************************************************************************
*
***************************************************************************/
void comp_net_us_ll(tus_ll *p,char *buf)
{
  char str[STR_LEN_LN + 1];
  register int x;

  sprintf(str,"%s %x\n",dtusll_flags,p->flags);
  strcat(buf,str);

  sprintf(str,"%s %s\n",dtusll_user,p->user);
  strcat(buf,str);

  sprintf(str,"%s %s\n",dtusll_email,p->email);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_tt_lim,p->tt_lim);
  strcat(buf,str);

  for(x = 0;x < DT_LIM_SIZE;x++)
  {
    sprintf(str,"%s %d %x %x %x %x %x\n",
      dtusll_dt_lim,
      x,
      p->dt_lim[x].d_fr,
      p->dt_lim[x].d_to,
      p->dt_lim[x].t_fr,
      p->dt_lim[x].t_to,
      p->dt_lim[x].limit);
    strcat(buf,str);
  }

  sprintf(str,"%s %x %x %x\n",
    dtusll_expiry_date,
    p->expiry_date.day,
    p->expiry_date.month,
    p->expiry_date.year);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_nice_factor,p->nice_factor);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_t_ul_lim,p->t_ul_lim);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_t_dl_lim,p->t_dl_lim);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_d_ul_lim,p->d_ul_lim);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_d_dl_lim,p->d_dl_lim);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtusll_act_timeout,p->act_timeout);
  strcat(buf,str);
}

void comp_net_us_hr(tus_hr *p,char *buf)
{
  char str[STR_LEN_LN + 1];

  sprintf(str,"%s %x\n",dtushr_flags,p->flags);
  strcat(buf,str);

  sprintf(str,"%s %s\n",dtushr_root_emailname,p->root_emailname);
  strcat(buf,str);

  sprintf(str,"%s %s\n",dtushr_email_host,p->email_host);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtushr_sleep_time,p->sleep_time);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtushr_grace_time,p->grace_time);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtushr_grace_volume,p->grace_volume);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtushr_save_time,p->save_time);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtushr_min_free_dialins,p->min_free_dialins);
  strcat(buf,str);

  comp_net_us_ll(&(p->default_us),buf);
}

/***************************************************************************
*
***************************************************************************/
void decomp_net_us_ll(tus_ll *p,char *ins)
{
  if(ins[0] != ':')
    return;

  if(!strncmp(ins,dtusll_flags,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->flags));
    return;
  }

  if(!strncmp(ins,dtusll_user,CMD_DT_SIZE))
  {
    decomp_str(p->user,ins);
    return;
  }

  if(!strncmp(ins,dtusll_email,CMD_DT_SIZE))
  {
    decomp_str(p->email,ins);
    return;
  }

  if(!strncmp(ins,dtusll_tt_lim,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->tt_lim));
    return;
  }

  if(!strncmp(ins,dtusll_dt_lim,CMD_DT_SIZE))
  {
    int x,a,b,c,d,e;

    sscanf(ins,"%*s %d %x %x %x %x %x",
              &x,&a,&b,&c,&d,&e);
    if(x >= DT_LIM_SIZE)
      return;
    if(x == 0)
    {
      register int i;

      for(i = 1;i < DT_LIM_SIZE;i++)
        p->dt_lim[i].limit = -1;
    }
    p->dt_lim[x].d_fr  = a;
    p->dt_lim[x].d_to  = b;
    p->dt_lim[x].t_fr  = c;
    p->dt_lim[x].t_to  = d;
    p->dt_lim[x].limit = e;
    return;
  }

  if(!strncmp(ins,dtusll_expiry_date,CMD_DT_SIZE))
  {
    int a,b;

    sscanf(ins,"%*s %x %x %x",&a,&b,&(p->expiry_date.year));
    p->expiry_date.day = a;
    p->expiry_date.month = b;
    return;
  }

  if(!strncmp(ins,dtusll_nice_factor,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->nice_factor));
    return;
  }

  if(!strncmp(ins,dtusll_t_ul_lim,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->t_ul_lim));
    return;
  }

  if(!strncmp(ins,dtusll_t_dl_lim,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->t_dl_lim));
    return;
  }

  if(!strncmp(ins,dtusll_d_ul_lim,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->d_ul_lim));
    return;
  }

  if(!strncmp(ins,dtusll_d_dl_lim,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->d_dl_lim));
    return;
  }

  if(!strncmp(ins,dtusll_act_timeout,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->act_timeout));
    return;
  }
}

void decomp_net_us_hr(tus_hr *p,char *ins)
{
  if(ins[0] != ':')
    return;

  if(!strncmp(ins,dtushr_flags,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->flags));
    return;
  }

  if(!strncmp(ins,dtushr_root_emailname,CMD_DT_SIZE))
  {
    decomp_str(p->root_emailname,ins);
    return;
  }

  if(!strncmp(ins,dtushr_email_host,CMD_DT_SIZE))
  {
    decomp_str(p->email_host,ins);
    return;
  }

  if(!strncmp(ins,dtushr_sleep_time,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->sleep_time));
    return;
  }

  if(!strncmp(ins,dtushr_grace_time,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->grace_time));
    return;
  }

  if(!strncmp(ins,dtushr_grace_volume,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->grace_volume));
    return;
  }

  if(!strncmp(ins,dtushr_save_time,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->save_time));
    return;
  }

  if(!strncmp(ins,dtushr_min_free_dialins,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->min_free_dialins));
    return;
  }

  decomp_net_us_ll(&(p->default_us),ins);
}

/***************************************************************************
*
***************************************************************************/
void comp_64b(char *cmd,_UD_64 val,char *buf)
{
  char str[STR_LEN_LN + 1];
  register unsigned int ms,ls;

  ms = val >> 32;
  ls = val & 0xffffffff;
  sprintf(str,"%s %x %x\n",cmd,ms,ls);
  strcat(buf,str);
}

void comp_net_um_ll(tum_ll *p,char *buf)
{
  char str[STR_LEN_LN + 1]; 
    
  sprintf(str,"%s %s\n",dtumll_user,p->user);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtumll_no_active,p->no_active);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtumll_last_active,p->last_active);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtumll_tt_used,p->tt_used);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtumll_dt_zone_limit,p->dt_zone_limit);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtumll_dt_set_limit,p->dt_set_limit);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dtumll_dt_used,p->dt_used);
  strcat(buf,str);

  comp_64b(dtumll_t_ul,p->t_ul,buf);
  comp_64b(dtumll_t_dl,p->t_dl,buf);
  comp_64b(dtumll_d_ul,p->d_ul,buf);
  comp_64b(dtumll_d_dl,p->d_dl,buf);

  sprintf(str,"%s %x\n",dtumll_tmp_flags,p->tmp_flags);
  strcat(buf,str);
}

/***************************************************************************
*
***************************************************************************/
void decomp_64b(_UD_64 *val,char *dst)
{
  unsigned int ms,ls;

  sscanf(dst,"%*s %x %x",&ms,&ls);
  *val = ((_UD_64)ms << 32) + (_UD_64)ls;
}

void decomp_net_um_ll(tum_ll *p,char *ins)
{
  if(ins[0] != ':')
    return;

  if(!strncmp(ins,dtumll_user,CMD_DT_SIZE))
  {
    decomp_str(p->user,ins);
    return;
  }

  if(!strncmp(ins,dtumll_no_active,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->no_active));
    return;
  }

  if(!strncmp(ins,dtumll_last_active,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->last_active));
    return;
  }

  if(!strncmp(ins,dtumll_tt_used,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->tt_used));
    return;
  }

  if(!strncmp(ins,dtumll_dt_zone_limit,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->dt_zone_limit));
    return;
  }

  if(!strncmp(ins,dtumll_dt_set_limit,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->dt_set_limit));
    return;
  }

  if(!strncmp(ins,dtumll_dt_used,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->dt_used));
    return;
  }

  if(!strncmp(ins,dtumll_t_ul,CMD_DT_SIZE))
  {
    decomp_64b(&(p->t_ul),ins);
    return;
  }

  if(!strncmp(ins,dtumll_t_dl,CMD_DT_SIZE))
  {
    decomp_64b(&(p->t_dl),ins);
    return;
  }

  if(!strncmp(ins,dtumll_d_ul,CMD_DT_SIZE))
  {
    decomp_64b(&(p->d_ul),ins);
    return;
  }

  if(!strncmp(ins,dtumll_d_dl,CMD_DT_SIZE))
  {
    decomp_64b(&(p->d_dl),ins);
    return;
  }

  if(!strncmp(ins,dtumll_tmp_flags,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->tmp_flags));
    return;
  }
}

/***************************************************************************
*
***************************************************************************/
void us_ll_clr(tus_ll *p)
{
  register int x;

  p->flags =
    TUSF_NOTIFY_VIA_EMAIL |
    TUSF_EXPT_TOTAL_TIME |
    TUSF_EXPT_DAILY_TIME |
    TUSF_EXPT_EXPIRY_DATE |
    TUSF_EXPT_NICE |
    TUSF_EXPT_TOTAL_UL_LIM |
    TUSF_EXPT_TOTAL_DL_LIM |
    TUSF_EXPT_DAILY_UL_LIM |
    TUSF_EXPT_DAILY_DL_LIM |
    TUSF_INSTANT_PURGE_DT |
    TUSF_CLOBBER_MULT_LOGINS |
    TUSF_EXPT_ACT_TIMEOUT;

  p->user[0] = 0;
  p->email[0] = 0;
  p->tt_lim = 0;
  for(x = 0;x < DT_LIM_SIZE;x++)
  {
    p->dt_lim[x].d_fr = -1;
    p->dt_lim[x].d_to = 0;
    p->dt_lim[x].t_fr = 0;
    p->dt_lim[x].t_to = 0;
    p->dt_lim[x].limit = 0;
  }
  p->expiry_date.day = 1;
  p->expiry_date.month = 1;
  p->expiry_date.year = 1970;
  p->nice_factor = 0;
  p->t_ul_lim = 0;
  p->t_dl_lim = 0;
  p->d_ul_lim = 0;
  p->d_dl_lim = 0;
  p->act_timeout = 30 * 60;
}

void us_hr_clr(tus_hr *p)
{
  p->flags = TUSF_HR_MIN_FREE_DIALIN_EXEMPT;
  TUSF_HR_S_RUN_LEVEL(p->flags,0);
  strcpy(p->root_emailname,"root");
  strcpy(p->email_host,"localhost");
  p->sleep_time = 60;
  p->grace_time = 5 * 60;
  p->grace_volume = 20;
  p->save_time = 15 * 60;
  p->min_free_dialins = 0;
  us_ll_clr(&(p->default_us));
}

/***************************************************************************
*
***************************************************************************/
void comp_net_tinfo_ll(ttinfo_ll *p,char *buf)
{
  char str[STR_LEN_LN + 1]; 

  sprintf(str,"%s %x\n",dttinfo_ll_type,p->type);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dttinfo_ll_pid,p->pid);
  strcat(buf,str);

  sprintf(str,"%s %s\n",dttinfo_ll_tty,p->tty);
  strcat(buf,str);

  sprintf(str,"%s %s\n",dttinfo_ll_nif,p->nif);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dttinfo_ll_ul,p->ul);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dttinfo_ll_dl,p->dl);
  strcat(buf,str);

  sprintf(str,"%s %x\n",dttinfo_ll_last_active,p->last_active);
  strcat(buf,str);
}

void decomp_net_tinfo_ll(ttinfo_ll *p,char *ins)
{
  if(ins[0] != ':')
    return;

  if(!strncmp(ins,dttinfo_ll_type,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->type));
    return;
  }

  if(!strncmp(ins,dttinfo_ll_pid,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->pid));
    return;
  }

  if(!strncmp(ins,dttinfo_ll_tty,CMD_DT_SIZE))
  {
    decomp_str(p->tty,ins);
    return;
  }

  if(!strncmp(ins,dttinfo_ll_nif,CMD_DT_SIZE))
  {
    decomp_str(p->nif,ins);
    return;
  }

  if(!strncmp(ins,dttinfo_ll_ul,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->ul));
    return;
  }

  if(!strncmp(ins,dttinfo_ll_dl,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->dl));
    return;
  }

  if(!strncmp(ins,dttinfo_ll_last_active,CMD_DT_SIZE))
  {
    sscanf(ins,"%*s %x",&(p->last_active));
    return;
  }
}
