/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 96,97,98,1999 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* the monitor, the heart of clobberd
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <utmp.h>
#include <pwd.h>
#include <dirent.h>
#include <sys/resource.h>
#include <sys/stat.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
#include "log.h"
#include "sm.h"
#include "notify.h"
#include "get_if_stats.h"
#include "clobber_user.h"
#include "net_server.h"

int running = 1,daemon_mode = 1,test_mode = 0;
int timer,old_timer,old_save_time,time_delta;
int no_free_dialins;
int modem_lenient_f = 0;

/***************************************************************************
*
***************************************************************************/
int user_exempt(char *user)
{
  register int x;

  for(x = 0;x < boot_st.exempt_user_size;x++)
    if(!strcmp(boot_st.exempt_user[x],user))
      return 1;
  return 0;
}

/***************************************************************************
* get shell stats (if a shell user)
***************************************************************************/
void get_shell_stats(ttinfo_ll *tinfo)
{
  struct stat stbuf;
  char str[STR_LEN];

  strcpy(str,"/dev/");
  strncat(str,tinfo->tty,STR_TTY_LEN);
  if(!stat(str,&stbuf))
  {
    /* If it's a login terminal, then all we need to do is
     * find out how active it is.
     */
    tinfo->type = TINFO_TYPE_LOGIN;
    if(stbuf.st_mtime > tinfo->last_active)
      tinfo->last_active = stbuf.st_mtime;
  }
}

/***************************************************************************
* remove any tinfo entries that are logged out of
***************************************************************************/
void check_defunc_tinfo(tum_ll *um)
{
  ttinfo_ll *p;

  p = um->tinfo_ll;
  while(p != NULL)
  {
    struct utmp *up;
    int match = 0;

    setutent();
    for(;;)
    {
      up = getutent();
      if(up == NULL)
        break;
      if(up->ut_type == USER_PROCESS)
        if(p->pid == up->ut_pid)
          match = 1;
    }
    endutent();
    if(!match)
    {
      char str[STR_LEN];
      ttinfo_ll *p_next;

      sprintf(str,"'%s' logoff from %s:%d",um->user,p->tty,p->pid);
      log_msg('I',str,1);
      p_next = p->next;
      remove_tinfo(um,p->pid);
      p = p_next;
    }
    else
      p = p->next;
  }
}

/***************************************************************************
* gathers terminal data of a given active user
***************************************************************************/
void stat_terminal(tum_ll *um,struct utmp *utmp_ptr)
{
  ttinfo_ll *tinfo;

  tinfo = find_tinfo(um,utmp_ptr->ut_pid);
  if(tinfo == NULL)
  {
    ttinfo_ll entry;
    char str[STR_LEN];

    entry.type = TINFO_TYPE_UNKNOWN;
    entry.pid = utmp_ptr->ut_pid;
    /*
     * Nb/ originally, i used tty-id (got from the utmp->ut_id field)
     * but pppd 2.3.8 ignores modifying this field.. therefore
     * we have to use the full utmp->ut_line field.
     */
    strncpy(entry.tty,utmp_ptr->ut_line,STR_TTY_LEN);
    entry.tty[STR_TTY_LEN] = 0;
    strcpy(entry.nif,none_label);
    entry.ul = 0;
    entry.dl = 0;
    entry.old_ul = -1;  /*when if stat'er sees -1 it will init it*/
    entry.old_dl = -1;
    entry.last_active = timer;
    append_tinfo(um,&entry);
    sprintf(str,"'%s' logon to %s:%d",um->user,entry.tty,entry.pid);
    log_msg('I',str,1);

    tinfo = find_tinfo(um,utmp_ptr->ut_pid);
  }

  if(tinfo != NULL)
  {
    tinfo->type = TINFO_TYPE_UNKNOWN;
    get_ppp_stats(tinfo,utmp_ptr); /*if any*/
    if(tinfo->type == TINFO_TYPE_UNKNOWN)
      get_shell_stats(tinfo);      /*if any*/
  }
}

/***************************************************************************
* gathers information about active users and stores it in um linked list
***************************************************************************/
void scan_utmp(void)
{
  struct utmp *utmp_ptr;
  char str[STR_LEN];
  int garbage_f = 0;

  void prep_um(tum_ll *um)
  {
    if(um != NULL)
    {
      um->no_active = 0;
      check_defunc_tinfo(um);
      prep_um(um->next);
    }
  }

  void purge_um(tum_ll *um)
  {
    if(um != NULL)
    {
      if(um->no_active == 0)
        init_um(um);
      purge_um(um->next);
    }
  }

  cache_nif();  /*for fast ppp lock lookups*/
  prep_um(um_ll);
  setutent();
  for(;;)
  {
    utmp_ptr = getutent();
    if(utmp_ptr == NULL)
      break;
    if(utmp_ptr->ut_type == USER_PROCESS)
    {
      char tn[STR_USER_LEN + 1];

      strncpy(tn,utmp_ptr->ut_user,STR_USER_LEN);
      tn[STR_USER_LEN] = 0;

      /*
       * find out if entry is a user in /etc/passwd. if not, then it's
       * garbage, and ignored.
       */
      if(getpwnam(tn) == NULL)
        garbage_f = 1;
      else
        if(!user_exempt(tn))
        {
          tum_ll *um;

          um = find_um(tn);
          if(um == NULL)
          {
            tum_ll u;

            strcpy(u.user,tn);
            u.tt_used = (timer - utmp_ptr->ut_time) - (timer - old_timer);
            u.dt_used = u.tt_used;
            u.t_ul = 0;
            u.t_dl = 0;
            u.d_ul = 0;
            u.d_dl = 0;
            u.no_active = 0;
            u.tinfo_ll = NULL;
            init_um(&u);
            append_um(&u);
            sprintf(str,"'%s' new user",u.user);
            log_msg('I',str,1);

            um = find_um(tn);
          }

          if(um != NULL)
          {
            (um->no_active)++;
            stat_terminal(um,utmp_ptr);
          }
        }
    }
  }
  endutent();
  purge_um(um_ll);
  free_nif();     /*free all network if (ppp locks) ll*/

  if(garbage_f)
    log_msg('W',"Garbage found in utmp file",0);
}

/***************************************************************************
* find number of free dialin terminals
***************************************************************************/
void dialins_free(void)
{
  no_free_dialins = -1;
  if(!(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT))
  {
    register int x;
    char str[STR_LEN];

    /*
     * to find the no of free modems, we compare how many "add_tty:"'s
     * are in the utmp file
     */
    no_free_dialins = boot_st.mfd_tty_t_size;
    for(x = 0;x < boot_st.mfd_tty_t_size;x++)
    {
      struct utmp *up;

      setutent();
      for(;;)
      {
        up = getutent();
        if(up == NULL)
          break;
        if(up->ut_type == USER_PROCESS)
          if(!strncmp(up->ut_line,boot_st.mfd_tty_t[x],STR_TTY_LEN))
          {
            no_free_dialins--;
            break;
          }
      }
      endutent();
    }
    if(no_free_dialins >= us_hr.min_free_dialins)
    {
      if(!modem_lenient_f)
      {
        modem_lenient_f = 1;
        sprintf(str,"Modem Lenancy on (Dialin Stat: %d/%d)",
          no_free_dialins,us_hr.min_free_dialins);
        log_msg('I',str,1);
      }
    }
    else
    {
      if(modem_lenient_f)
      {
        modem_lenient_f = 0;
        sprintf(str,"Modem Lenancy off (Dialin Stat: %d/%d)",
          no_free_dialins,us_hr.min_free_dialins);
        log_msg('I',str,1);
      }
    }
  }
  else
    if(modem_lenient_f)
    {
      modem_lenient_f = 0;
      log_msg('I',"Modem Lenancy exempt",1);
    }
}

/***************************************************************************
* now we aggragate all the tinfo volumes. NB/ this must be done while
* um record not locked.
***************************************************************************/
void tally_nif_volumes(tum_ll *um)
{
  ttinfo_ll *p;
  int ul = 0,dl = 0,nul,ndl;

  um->last_active = 0;
  p = um->tinfo_ll;
  while(p != NULL)
  {
    ul += p->ul;
    dl += p->dl;
    /*take the most active terminal*/
    if(p->last_active > um->last_active)
      um->last_active = p->last_active;
    p = p->next;
  }
  nul = ul - um->old_ul;
  ndl = dl - um->old_dl;
  um->old_ul = ul;
  um->old_dl = dl;
  (um->t_ul) += nul;
  (um->t_dl) += ndl;
  (um->d_ul) += nul;
  (um->d_dl) += ndl;
}

/***************************************************************************
*
***************************************************************************/
void mon_entry(tus_ll *us,tum_ll *um)
{
  struct tm *tm_s;

  void log_clobber_user(void)
  {
    char str[STR_LEN];

    sprintf(str,"'%s' End of Grace period. Disconnecting",um->user);
    log_msg('I',str,1);
    clobber_user(um);
  }

  tm_s = localtime((time_t *)&timer);
  if(tm_s != NULL)
  {
    char str[STR_LEN];
    register int x;

/*clobber if clobber flag set (user is banned)*/
    if(us->flags & TUSF_CLOBBER)
    {
      notify_user(us,um,CLOB_INDEX);
      sprintf(str,"'%s' banned, Disconnecting",um->user);
      log_msg('I',str,1);
      clobber_user(um);
      return;
    }

/*check for multiple logins*/
    if((um->no_active > 1) && (us->flags & TUSF_CLOBBER_MULT_LOGINS))
    {
      notify_user(us,um,CLOB_MULTIPLE_INDEX);
      sprintf(str,"'%s' Mult. logins. Flashed User, Disconnecting",
        um->user);
      log_msg('I',str,1);
      clobber_user(um);
      return;
    }

/*check expiry date*/
    if(!(us->flags & TUSF_EXPT_EXPIRY_DATE))
    {
      register unsigned int a,b;

      a = us->expiry_date.day +
          (us->expiry_date.month * 31) +
          (us->expiry_date.year * 31 * 12);
      b = tm_s->tm_mday +
          ((tm_s->tm_mon + 1) * 31) +
          ((tm_s->tm_year + 1900) * 31 * 12);
      if(b >= a)
      {
        notify_user(us,um,ACCT_EXPIRED_INDEX);
        sprintf(str,"'%s' Account Expired. Flashed User, Disconnecting",
          um->user);
        log_msg('I',str,1);
        clobber_user(um);
        return;
      }
    }

/*check if user is still active*/
    if(!(us->flags & TUSF_EXPT_ACT_TIMEOUT))
      if((timer - um->last_active) >= us->act_timeout)
      {
        notify_user(us,um,UACT_TIMEOUT_INDEX);
        sprintf(str,"'%s' User Activity Timeout. Flashed User, Disconnecting",
          um->user);
        log_msg('I',str,1);
        clobber_user(um);
        return;
      }

/*set niceness*/
    if(!(us->flags & TUSF_EXPT_NICE))
    {
      struct passwd *pw;

      pw = getpwnam(um->user);
      if(pw != NULL)
        setpriority(PRIO_USER,pw->pw_uid,us->nice_factor);
    }

/*set time zone*/
    if(!(us->flags & TUSF_EXPT_DAILY_TIME))
    {
      int dct[] = {6,0,1,2,3,4,5};

      um->dt_zone_limit = -1;
      um->dt_set_limit = 0;
      for(x = (DT_LIM_SIZE - 1);x >= 0;x--)
        if(us->dt_lim[x].d_fr != -1)
        {
          register int pass = 0;

          if((dct[tm_s->tm_wday] >= us->dt_lim[x].d_fr) &&
             (dct[tm_s->tm_wday] <= us->dt_lim[x].d_to))
            pass++;
          else
            if(us->dt_lim[x].d_fr > us->dt_lim[x].d_to)
              pass++;
          if((tm_s->tm_hour >= us->dt_lim[x].t_fr) &&
             (tm_s->tm_hour <= us->dt_lim[x].t_to))
            pass++;
          else
            if(us->dt_lim[x].t_fr > us->dt_lim[x].t_to)
              pass++;
          if(pass == 2)
          {
            um->dt_zone_limit = x;
            um->dt_set_limit = us->dt_lim[um->dt_zone_limit].limit;
            break;
          }
        }
    }

/*check total time*/
    if(!(us->flags & TUSF_EXPT_TOTAL_TIME))
    {
      if(um->tt_used >= us->tt_lim)
      {
        log_clobber_user();
        return;
      }
      if(!(um->tmp_flags & TUMF_TT_GRACED))
        if(um->tt_used >= (us->tt_lim - us_hr.grace_time))
        {
          notify_user(us,um,EXCEEDED_TT_INDEX);
          sprintf(str,"'%s' Total Time grace. Flashed User",um->user);
          log_msg('I',str,1);
          um->tmp_flags |= TUMF_TT_GRACED;
        }
    }

/*check daily time*/
    if(!(us->flags & TUSF_EXPT_DAILY_TIME))
      if(no_free_dialins < us_hr.min_free_dialins)
      {
        if(um->dt_used >= um->dt_set_limit)
        {
          log_clobber_user();
          return;
        }
        if(!(um->tmp_flags & TUMF_DT_GRACED))
          if(um->dt_used >= (um->dt_set_limit - us_hr.grace_time))
          {
            notify_user(us,um,EXCEEDED_DT_INDEX);
            sprintf(str,"'%s' Daily Time grace. Flashed User",um->user);
            log_msg('I',str,1);
            um->tmp_flags |= TUMF_DT_GRACED;
          }
      }

/*check total uploads limit*/
    if(!(us->flags & TUSF_EXPT_TOTAL_UL_LIM))
    {
      register int total_uploads;

      total_uploads = rnd2meg64(um->t_ul);
      if(total_uploads >= us->t_ul_lim)
      {
        log_clobber_user();
        return;
      }
      if(!(um->tmp_flags & TUMF_TUL_GRACED))
        if(total_uploads >= (us->t_ul_lim - us_hr.grace_volume))
        { 
          notify_user(us,um,EXCEEDED_TNET_INDEX);
          sprintf(str,"'%s' Total Uploads grace. Flashed User",
            um->user);
          log_msg('I',str,1);
          um->tmp_flags |= TUMF_TUL_GRACED;
        }
    }

/*check total dnloads limit*/
    if(!(us->flags & TUSF_EXPT_TOTAL_DL_LIM))
    {
      register int total_dnloads;

      total_dnloads = rnd2meg64(um->t_dl);
      if(total_dnloads >= us->t_dl_lim)
      { 
        log_clobber_user();
        return;
      }
      if(!(um->tmp_flags & TUMF_TUD_GRACED))
        if(total_dnloads >= (us->t_dl_lim - us_hr.grace_volume))
        {
          notify_user(us,um,EXCEEDED_TNET_INDEX);
          sprintf(str,"'%s' Total Downloads grace. Flashed User",
            um->user);
          log_msg('I',str,1);
          um->tmp_flags |= TUMF_TUD_GRACED;
        }
    }

/*check daily uploads limit*/
    if(!(us->flags & TUSF_EXPT_DAILY_UL_LIM))
    {
      register int daily_uploads;

      daily_uploads = rnd2meg64(um->d_ul);
      if(daily_uploads >= us->d_ul_lim)
      {
        log_clobber_user();
        return;
      }
      if(!(um->tmp_flags & TUMF_DUL_GRACED))
        if(daily_uploads >= (us->d_ul_lim - us_hr.grace_volume))
        { 
          notify_user(us,um,EXCEEDED_DNET_INDEX);
          sprintf(str,"'%s' Daily Uploads grace. Flashed User",
            um->user);
          log_msg('I',str,1);
          um->tmp_flags |= TUMF_DUL_GRACED;
        }
    }

/*check daily dnloads limit*/
    if(!(us->flags & TUSF_EXPT_DAILY_DL_LIM))
    {
      register int daily_dnloads;

      daily_dnloads = rnd2meg64(um->d_dl);
      if(daily_dnloads >= us->d_dl_lim)
      {
        log_clobber_user();
        return;
      }
      if(!(um->tmp_flags & TUMF_DUD_GRACED))
        if(daily_dnloads >= (us->d_dl_lim - us_hr.grace_volume))
        {
          notify_user(us,um,EXCEEDED_DNET_INDEX);
          sprintf(str,"'%s' Daily Downloads grace. Flashed User",
            um->user);
          log_msg('I',str,1);
          um->tmp_flags |= TUMF_DUD_GRACED;
        }
    }
  }
}

void monitor(void)
{
  tum_ll *um;
  register int lf = 0;

  if(us_hr_locked())
  {
    log_msg('W',"us_hr locked. monitoring ignored",1);
    lf = 1;
  }
  dialins_free();
  um = um_ll;
  for(;;)
  {
    if(um == NULL)
      break;
    if(user_locked(um->user))
    {
      char str[STR_LEN];

      sprintf(str,"'%s' locked. monitoring for user ignored",um->user);
      log_msg('W',str,1);
    }
    else
    {
      tus_ll *us;

      tally_nif_volumes(um); /*now would be a good time to tally nif volumes*/
      us = find_us2(um->user);
      if(um->no_active > 0)
      {
        um->tt_used += time_delta;
        um->dt_used += time_delta;
        if(!lf)
        {
          if(us != NULL)
            mon_entry(us,um);
          else
            um->dt_zone_limit = -1;
        }
      }
      else
      {
        if(us != NULL)
          if(us->flags & TUSF_INSTANT_PURGE_DT)
          {
            um->dt_used = 0;
            um->d_ul = 0;
            um->d_dl = 0;
          }
      }
    }
    um = um->next;
  }
}
