/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996-1999 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* network server (responds to clobberedit and clobbertool sessions over
* internet)
*
* notes about structure locking..
* -------------------------------
*
* locking is needed to arbitrate who can modify data during given time
* frame. the rule is that only one thing can modify a record at a time. if
* you get lots of different things reading/modifying a structure, data will
* become corrupt -- see database theory.
*
* in the daemon itself, when a structure is locked, it won't touch it, as
* the data being locked is still undergoing modification, and it's contents
* while locked is meaningless.
*
* from a network client point of view, the locking flags are merely
* semiphores, and any client (if programmed) can willfully modify structures
* even if they are locked. clobberedit and clobbertool won't  do this
* of course, as they will lock any structures before modifying them.
*
* why haven't i enforced locking?-
*
*  - code like this is very light weight, so design was very simple.
*  - 99% of users will only operate with one clobberedit session. So, the
*    condition almost never arises anyway.
*  - the sementics are within clobberd- there's no third party programs
*    that will talk to it, and it's unlikely there will be any.
*  - you have to be an authorised connection anyway.
*
* therefore, i don't see it necessary to make a complex arbitrator
* to manage multiple clients sessions.
*
* if the need arises in future, then parts of this file will have to
* be modified to deny clients access to structures if any are locked.
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
#include "log.h"
#include "sm.h"
#include "monitor.h"

int  addrlen = sizeof(struct sockaddr_in);
int  listen_s;
tkon kon[MAX_KONS];
int  no_kons = 0;
char recv_buf[RECV_BUF_LEN];
int  recv_buf_ptr = RECV_BUF_LEN,recv_buf_size = 0;

/***************************************************************************
* Server specific net functions
***************************************************************************/
void s_close_kon(tkon *k)
{
  char str[STR_LEN];

  sprintf(str,"[%s:%d] END CONNECTION",k->hostname,k->port);
  log_msg('I',str,1);
  close_socket(k->s);
  k->s = -1;
  no_kons--;
}

int s_net_readln(tkon *k,char *str,int max_str_len)
{
  register int str_ptr = 0;

  for(;;)
    if(recv_buf_ptr < recv_buf_size)
    {
      if(recv_buf[recv_buf_ptr] == '\n')
      {
        register int y;

        str[str_ptr] = 0;
        if(str_ptr > 1)
        {
          y = str_ptr - 1;
          if(str[y] == '\r')
            str[y] = 0;
        }
        recv_buf_ptr++;
        return 1;
      }
      else
      {
        str[str_ptr] = recv_buf[recv_buf_ptr];
        recv_buf_ptr++;
        if(str_ptr < max_str_len)
          str_ptr++;
        else
        {
          str[str_ptr] = 0;
          return 1;
        }
      }
    }
    else
    {
      fd_set fds;
      struct timeval tv = {0,0};

      FD_ZERO(&fds);
      FD_SET(k->s,&fds);
      switch(select(k->s + 1,&fds,NULL,NULL,&tv))
      {
        case -1:
          s_close_kon(k);
        case 0:
          return 0;
        default:
          recv_buf_ptr = 0;
          recv_buf_size = recv(k->s,recv_buf,RECV_BUF_LEN,0);
          switch(recv_buf_size)
          {
            case 0:
              s_close_kon(k);
            case -1:
              return 0;
          }
          break;
      }
    }
}

int s_net_write(tkon *k,char *str)
{
  register int sl;
            
  sl = strlen(str);
  if(send(k->s,str,sl,0) == sl)
    return 1;
  return 0;
}

int s_net_write_cmd(tkon *k,char *str)
{
  register int sl;
            
  sl = strlen(str);
  if(send(k->s,str,sl,0) == sl)
    if(send(k->s,"\n",1,0) == 1)
      return 1;
  return 0;
}

void kill_all_kons(void)
{
  register int a;

  for(a = 0;a < MAX_KONS;a++)
    if(kon[a].s != -1)
      s_close_kon(&(kon[a]));
  close_socket(listen_s);
}

/***************************************************************************
*
***************************************************************************/
void check_new_kons(void)
{
  register int kon_no;

  for(kon_no = 0;kon_no < MAX_KONS;kon_no++)
    if(kon[kon_no].s == -1)
    {
      register int sr;
      fd_set fds;
      struct timeval tv = {0,0};

      FD_ZERO(&fds);
      FD_SET(listen_s,&fds);
      if((sr = select(listen_s + 1,&fds,NULL,NULL,&tv)) == -1)
      {
        log_error("select(), aborting check_new_kons()");
        running = 0;
      }
      else
        if(sr > 0)
        {
          register int cs;
          struct sockaddr_in client_addr_in;
          char str[STR_LEN];

          cs = accept(listen_s,(struct sockaddr *)&client_addr_in,&addrlen);
          if(cs == -1)
          {
            log_error("accept(), aborting check_new_kons()");
            running = 0;
          }
          else
          {
            struct linger ling;
            struct hostent *hp = NULL;

            ling.l_onoff  = 1;
            ling.l_linger = 1;
            setsockopt(cs,SOL_SOCKET,SO_LINGER,&ling,sizeof(ling));
            no_kons++;

            kon[kon_no].flags = 0;
            kon[kon_no].mode = TKON_MODE_CMD;
            kon[kon_no].s = cs;
#ifdef RESOLVE_CONNECTIONS
            hp = gethostbyaddr(
              (char *)&client_addr_in.sin_addr,
              sizeof(struct in_addr),
              client_addr_in.sin_family);
#endif
            if(hp == NULL)
              strcpy(kon[kon_no].hostname,
                inet_ntoa(client_addr_in.sin_addr));
            else
              strcpy(kon[kon_no].hostname,hp->h_name);
            kon[kon_no].port = ntohs(client_addr_in.sin_port);
            kon[kon_no].time = timer;
            kon[kon_no].user_lock[0] = 0;

            sprintf(str,"[%s:%d] NEW CONNECTION",
              kon[kon_no].hostname,kon[kon_no].port);
            log_msg('I',str,1);
            s_net_write(&(kon[kon_no]),
              "Clobberd " VERSION ", By Jason Nunn <jsno@downunder.net.au>\n");
            s_net_write_cmd(&(kon[kon_no]),cmd_ok);
          }
        }
    }
}

/***************************************************************************
*
***************************************************************************/
char *net_get_param(char *cmd,char *str)
{
  register int si;

  si = strlen(cmd) + 1;
  if(strlen(str) > si)
    return str + si;
  return NULL;
}

/***************************************************************************
*
***************************************************************************/
void auth(tkon *k,char *param)
{
  char str[STR_LEN];

  void fail(void)
  {
    s_net_write_cmd(k,cmd_er);
    sprintf(str,"[%s:%d] cmd %s (failed)",k->hostname,k->port,cmd_pass);
    log_msg('I',str,1);
  }
  if(param != NULL)
    if(!strcmp(boot_st.passwd,param))
    {
      k->flags |= TKON_AUTHORISED;
      s_net_write_cmd(k,cmd_ok);
      sprintf(str,"[%s:%d] cmd %s (authorised)",k->hostname,k->port,cmd_pass);
      log_msg('I',str,0);
    }
    else
      fail();
  else
    fail();
}

/***************************************************************************
*
***************************************************************************/
void get_stats(tkon *k)
{
  char str[STR_LEN];
  register int x;

  for(x = 0;x < MAX_KONS;x++)
    if(kon[x].s != -1)
    {
      sprintf(str,":%-40s %6d %s\n",
        kon[x].hostname,
        kon[x].port,
        kon[x].flags & TKON_AUTHORISED ? "auth" : "");
      s_net_write(k,str);
    }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_stat);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_us_hr(tkon *k)
{
  char str[STR_BIG] = "";

  comp_net_us_hr(&us_hr,str);
  s_net_write(k,str);
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gushr);
  log_msg('I',str,0);  
}

void lock_us_hr(tkon *k)
{
  char str[STR_LEN];

  if(!us_hr_locked())
  {
    k->flags |= TKON_US_HR_LOCKED;
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_lushr);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_us_users(tkon *k)
{
  char str[STR_LEN];
  tus_ll *p;

  p = us_ll;
  while(p != NULL)
  {
    sprintf(str,":%s\n",p->user);
    s_net_write(k,str);
    p = p->next;
  }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gusu);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_us(tkon *k,char *param,int type)
{
  char str[STR_BIG] = "";

  if(param != NULL)
  {
    tus_ll *us;

    if(type)
      us = find_us2(param);
    else
      us = find_us(param);
    if(us != NULL)
    {
      comp_net_us_ll(us,str);
      s_net_write(k,str);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",
    k->hostname,k->port,type ? cmd_gus2 : cmd_gus);
  log_msg('I',str,0);
}

void lock_us(tkon *k,char *param)
{
  char str[STR_LEN];

  if(param != NULL)
  {
    if(!user_locked(param))
    {
      strcpy(k->user_lock,param);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_luser);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void new_us(tkon *k,char *param)
{
  char str[STR_LEN];
      
  if(param != NULL)
  {
    if(find_us(param) == NULL)
    {
      strcpy(k->user_lock,param);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_nus);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void delete_us(tkon *k,char *param)
{
  char str[STR_LEN];
      
  if(param != NULL)
  {
    remove_us(param);
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_dus);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_um_users(tkon *k)
{
  char str[STR_LEN];
  tum_ll *p;
   
  p = um_ll;
  while(p != NULL)
  {
    sprintf(str,":%s\n",p->user);
    s_net_write(k,str);
    p = p->next;
  }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gumu);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void get_um(tkon *k,char *param)
{
  char str[STR_BIG] = "";

  if(param != NULL)
  {
    tum_ll *um;

    um = find_um(param);
    if(um != NULL)
    {
      comp_net_um_ll(um,str);
      s_net_write(k,str);
      s_net_write_cmd(k,cmd_ok);
    }
    else
      s_net_write_cmd(k,cmd_er);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gum);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void delete_um(tkon *k,char *param)
{
  char str[STR_LEN];
      
  if(param != NULL)
  {
    remove_um(param);
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_dum);
  log_msg('I',str,0);
}

/***************************************************************************
*
***************************************************************************/
void do_report(tkon *k)
{
  FILE *fp;
  char str[STR_LEN];

  if((fp = fopen(REPORT_FILE,"w")) != NULL)
  {
    tum_ll *um;
    int total_users = 0,
        total_ttime = 0,
        total_dtime = 0;
    double total_tul = 0,
           total_tdl = 0,
           total_dul = 0,
           total_ddl = 0;

    fprintf(fp,"Clobberd " VERSION " Report."
               " %90s\n",
      ctime((time_t *)&timer));
    fprintf(fp,
      "User                             "
      "    T Time "
      " T U/lds (Mb) "
      " T D/lds (Mb) "
      "    D Time "
      " D U/lds (Mb) "
      " D D/lds (Mb) \n\n");
    um = um_ll;
    for(;;)
    {
      double x;

      if(um == NULL)
        break;

      fprintf(fp,"%-32s ",um->user);

      str_hh_mm(um->tt_used,str);
      fprintf(fp,"%10s ",str);
      total_ttime += um->tt_used;

      x = rnd2meg64(um->t_ul);
      total_tul += x;
      fprintf(fp,"%13.2f ",x);

      x = rnd2meg64(um->t_dl);
      total_tdl += x;
      fprintf(fp,"%13.2f ",x);

      str_hh_mm(um->dt_used,str);
      fprintf(fp,"%10s ",str);
      total_dtime += um->dt_used;

      x = rnd2meg64(um->d_ul);
      total_dul += x;
      fprintf(fp,"%13.2f ",x);

      x = rnd2meg64(um->d_dl);
      total_ddl += x;
      fprintf(fp,"%13.2f",x);

      fprintf(fp,"\n");
      total_users++;
      um = um->next;
    }

    fprintf(fp,"\n%-32d ",total_users);

    str_hh_mm(total_ttime,str);
    fprintf(fp,"%10s ",str);
    fprintf(fp,"%13.2f ",total_tul);
    fprintf(fp,"%13.2f ",total_tdl);

    str_hh_mm(total_dtime,str);
    fprintf(fp,"%10s ",str);
    fprintf(fp,"%13.2f ",total_dul);
    fprintf(fp,"%13.2f",total_ddl);
    fprintf(fp,"\n");

    fclose(fp);
  }
  else
  {
    sprintf(str,"[%s:%d] Error printing report",k->hostname,k->port);
    log_msg('I',str,1);
  }
}

void total_purge(tkon *k)
{
  char str[STR_LEN];

  do_report(k);
  free_um();
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_tpurg);
  log_msg('I',str,1);
}

void daily_purge(tkon *k)
{
  tum_ll *um;
  char str[STR_LEN];

  um = um_ll;
  for(;;)
  {
    if(um == NULL)
      break;
    um->dt_used = 0;
    um->d_ul = 0;
    um->d_dl = 0;
    um = um->next;
  }
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_dpurg);
  log_msg('I',str,1);
}

void report(tkon *k)
{
  char str[STR_LEN];

  do_report(k);
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_trept);
  log_msg('I',str,1);
}

/***************************************************************************
*
***************************************************************************/
void user_stat(tkon *k,char *param)
{
  char str[STR_LEN];

  void pe(void)
  {
    s_net_write(k,":No such user.\n");
  }

  if(param != NULL)
  {
    tus_ll *us;
    tum_ll *um;

    us = find_us2(param);
    um = find_um(param);
    if(um != NULL)
    {
      char str1[STR_LEN],str2[STR_LEN],*str3,str4[STR_LEN];
      double i;

      void na_strs(void)
      {
        strcpy(str1,na_label);
        strcpy(str2,na_label);
      }

      void exempt_strs(void)
      {
        strcpy(str1,exempt_label);
        strcpy(str2,exempt_label);
      }

      double diff_v(double lim,double used)
      {
        double x;
        
        x = lim - used;
        if(x < 0)
          x = 0;
        return x;
      }

      sprintf(str,
        ":User                : %s\n",um->user);
      s_net_write(k,str);

      str3 = ctime((time_t *)&(um->last_active));
      if(str3 != NULL)
      {
      sprintf(str,
        ":Last On             : %s",str3);
        s_net_write(k,str);
      }

      str_hh_mm(us_hr.grace_time,str2);
      sprintf(str,
        ":Grace Time          : %s\n",str2);
      s_net_write(k,str);

      sprintf(str,
        ":Grace Volume (Mb)   : %d\n",us_hr.grace_volume);
      s_net_write(k,str);

      if(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT)
        strcpy(str2,exempt_label);
      else
        sprintf(str2,"%d",us_hr.min_free_dialins);
      sprintf(str,
        ":Minimum Free Dialins: %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_ACT_TIMEOUT)
          strcpy(str2,exempt_label);
        else
          str_hh_mm(us->act_timeout,str2);
      sprintf(str,
        ":Activity Timeout    : %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_EXPIRY_DATE)
          strcpy(str2,exempt_label);
        else
          compose_date(&(us->expiry_date),str2);
      sprintf(str,
        ":Expiry Date         : %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        if(us->flags & TUSF_EXPT_NICE)
          strcpy(str2,exempt_label);
        else
          sprintf(str2,"%d",us->nice_factor);
      sprintf(str,
        ":Niceness            : %s\n",str2);
      s_net_write(k,str);

/* TT used*/
      str_hh_mm(um->tt_used,str4);
      if(us == NULL)
        na_strs();
      else
      {
        if(us->flags & TUSF_EXPT_TOTAL_TIME)
          exempt_strs();
        else
        {
          str_hh_mm(diff_v(us->tt_lim,um->tt_used),str1);
          str_hh_mm(us->tt_lim,str2);
        }
      }
      sprintf(str,
        ":Total Time (hhh:mm) : %s (Left: %s) (Lim: %s)\n",
        str4,str1,str2);
      s_net_write(k,str);

/* TD used*/
      str_hh_mm(um->dt_used,str4);
      if(us == NULL)
        na_strs();
      else
        if(us->flags & TUSF_EXPT_DAILY_TIME)
          exempt_strs();
        else
        {
          if(um->dt_zone_limit == -1)
            na_strs();
          else
          {
            str_hh_mm(diff_v(um->dt_set_limit,um->dt_used),str1);
            str_hh_mm(um->dt_set_limit,str2);
          }
        }
      sprintf(str,
        ":Daily Time (hhh:mm) : %s (Left: %s) (Lim: %s)\n",
        str4,str1,str2);
      s_net_write(k,str);

/*t_ul*/
      i = rnd2meg64(um->t_ul);
      if(us == NULL)
        na_strs();
      else
        if(us->flags & TUSF_EXPT_TOTAL_UL_LIM)
          exempt_strs();
        else
        {
          sprintf(str1,"%.2f",diff_v(us->t_ul_lim,i));
          sprintf(str2,"%d",us->t_ul_lim);
        }
      sprintf(str,
        ":Total Uploads (Mb)  : %.2f (Left: %s) (Lim: %s)\n",
        i,str1,str2);
      s_net_write(k,str);

/*t_dl*/
      i = rnd2meg64(um->t_dl);
      if(us == NULL)
        na_strs();
      else
        if(us->flags & TUSF_EXPT_TOTAL_DL_LIM)
          exempt_strs();
        else
        {
          sprintf(str1,"%.2f",diff_v(us->t_dl_lim,i));
          sprintf(str2,"%d",us->t_dl_lim);
        }
      sprintf(str,
        ":Total Dnloads (Mb)  : %.2f (Left: %s) (Lim: %s)\n",
        i,str1,str2);
      s_net_write(k,str);

/*d_ul*/
      i = rnd2meg64(um->d_ul);
      if(us == NULL)
        na_strs();
      else
        if(us->flags & TUSF_EXPT_DAILY_UL_LIM)
          exempt_strs();
        else
        {
          sprintf(str1,"%.2f",diff_v(us->d_ul_lim,i));
          sprintf(str2,"%d",us->d_ul_lim);
        }
      sprintf(str,
        ":Daily Uploads (Mb)  : %.2f (Left: %s) (Lim: %s)\n",
        i,str1,str2);
      s_net_write(k,str);

/*d_dl*/
      i = rnd2meg64(um->d_dl);
      if(us == NULL)
        na_strs();
      else
        if(us->flags & TUSF_EXPT_DAILY_DL_LIM)
          exempt_strs();
        else
        {
          sprintf(str1,"%.2f",diff_v(us->d_dl_lim,i));
          sprintf(str2,"%d",us->d_dl_lim);
        }
      sprintf(str,
        ":Daily Dnloads (Mb)  : %.2f (Left: %s) (Lim: %s)\n",
        i,str1,str2);
      s_net_write(k,str);

/**/
      if(us == NULL)
        strcpy(str2,na_label);
      else
        strcpy(str2,us->flags & TUSF_INSTANT_PURGE_DT ? yes_label : no_label);
      sprintf(str,
        ":Purge Daily Settings: %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        strcpy(str2,us->flags & TUSF_CLOBBER_MULT_LOGINS ? yes_label : no_label);
      sprintf(str,
        ":Kill Multiple logins: %s\n",str2);
      s_net_write(k,str);

      if(us == NULL)
        strcpy(str2,na_label);
      else
        strcpy(str2,us->flags & TUSF_CLOBBER ? yes_label : no_label);
      sprintf(str,
        ":Banned              : %s\n",str2);
      s_net_write(k,str);
    }
    else
      pe();
  }
  else
    pe();
  s_net_write_cmd(k,cmd_ok);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_ustat);
  log_msg('I',str,0);
}

/***************************************************************************
* Get (request) tinfo struct
***************************************************************************/
void get_tinfo(tkon *k,char *user)
{
  char str[STR_BIG];
  tum_ll *um;

  um = find_um(user);
  if(um != NULL)
  {
    ttinfo_ll *p;

    p = um->tinfo_ll;
    while(p != NULL)
    {
      s_net_write(k,":@\n");
      str[0] = 0;
      comp_net_tinfo_ll(p,str);
      s_net_write(k,str);
      s_net_write(k,":#\n");
      p = p->next;
    }
    s_net_write_cmd(k,cmd_ok);
  }
  else
    s_net_write_cmd(k,cmd_er);
  sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_gtinf);
  log_msg('I',str,0);  
}

/***************************************************************************
*
***************************************************************************/
void cmd_mode(tkon *k,char *ins)
{
  char str[STR_LEN];

  if(!strcmp(ins,cmd_quit))
  {
    sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_quit);
    log_msg('I',str,0);
    s_net_write_cmd(k,cmd_ok);
    s_close_kon(k);
  }
  else if(!strncmp(ins,cmd_pass,strlen(cmd_pass)))
    auth(k,net_get_param(cmd_pass,ins));
  else if(!strncmp(ins,cmd_ustat,strlen(cmd_ustat)))
    user_stat(k,net_get_param(cmd_ustat,ins));
  else
  {
    register int unk = 0;

    if(k->flags & TKON_AUTHORISED)
    {
      if(!strcmp(ins,cmd_stat))
        get_stats(k);
/*hr*/
      else if(!strcmp(ins,cmd_gushr))
        get_us_hr(k);
      else if(!strcmp(ins,cmd_sushr))
      {
        k->mode = TKON_MODE_S_US_HR;
        sprintf(str,"[%s:%d] cmd %s (start)",k->hostname,k->port,cmd_sushr);
        log_msg('I',str,0);
      }
      else if(!strcmp(ins,cmd_lushr))
        lock_us_hr(k);
      else if(!strcmp(ins,cmd_uushr))
      {
        k->flags &= ~TKON_US_HR_LOCKED;
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_uushr);
        log_msg('I',str,0);
      }
/*us*/
      else if(!strcmp(ins,cmd_gusu))
        get_us_users(k);

      else if(!strncmp(ins,cmd_gus2,strlen(cmd_gus2)))
        get_us(k,net_get_param(cmd_gus2,ins),1);

      else if(!strncmp(ins,cmd_gus,strlen(cmd_gus)))
        get_us(k,net_get_param(cmd_gus,ins),0);

      else if(!strcmp(ins,cmd_sus))
      {
        k->mode = TKON_MODE_S_US;
        sprintf(str,"[%s:%d] cmd %s (start)",k->hostname,k->port,cmd_sus);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_luser,strlen(cmd_luser)))
        lock_us(k,net_get_param(cmd_luser,ins));
      else if(!strcmp(ins,cmd_uuser))
      {
        k->user_lock[0] = 0;
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_uuser);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_nus,strlen(cmd_nus)))
        new_us(k,net_get_param(cmd_nus,ins));
      else if(!strncmp(ins,cmd_dus,strlen(cmd_dus)))
        delete_us(k,net_get_param(cmd_dus,ins));
/*um*/
      else if(!strcmp(ins,cmd_gumu))
        get_um_users(k);
      else if(!strncmp(ins,cmd_gum,strlen(cmd_gum)))
        get_um(k,net_get_param(cmd_gum,ins));
      else if(!strcmp(ins,cmd_sum))
      {
        k->mode = TKON_MODE_S_UM;
        sprintf(str,"[%s:%d] cmd %s (start)",k->hostname,k->port,cmd_sum);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_dum,strlen(cmd_dum)))
        delete_um(k,net_get_param(cmd_dum,ins));
/****/
      else if(!strcmp(ins,cmd_stime))
      {
        sprintf(str,":%x %x\n",us_hr.sleep_time,timer);
        s_net_write(k,str);
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s",k->hostname,k->port,cmd_stime);
        log_msg('I',str,0);
      }
      else if(!strcmp(ins,cmd_tpurg))
        total_purge(k);
      else if(!strcmp(ins,cmd_dpurg))
        daily_purge(k);
      else if(!strcmp(ins,cmd_trept))
        report(k);
      else if(!strcmp(ins,cmd_die))
      {
        running = 0;
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s, Server ending",
          k->hostname,k->port,cmd_die);
        log_msg('I',str,0);
      }
      else if(!strcmp(ins,cmd_save))
      {
        save_data_file();
        s_net_write_cmd(k,cmd_ok);
        sprintf(str,"[%s:%d] cmd %s, Saved datafile (forced)",
          k->hostname,k->port,cmd_save);
        log_msg('I',str,0);
      }
      else if(!strncmp(ins,cmd_gtinf,strlen(cmd_gtinf)))
        get_tinfo(k,net_get_param(cmd_gtinf,ins));
      else
        unk = 1;
    }
    else
      unk = 1;
    if(unk)
    {
      s_net_write_cmd(k,cmd_er);
      sprintf(str,"[%s:%d] cmd unknown",k->hostname,k->port);
      log_msg('I',str,0);
    }
  }
}

/***************************************************************************
*
***************************************************************************/
tus_hr tmp_us_hr;

void set_us_hr_mode(tkon *k,char *ins)
{
  if(!strcmp(ins,cmd_ok))
  {
    memcpy(&us_hr,&tmp_us_hr,sizeof(tus_hr));
    k->mode = TKON_MODE_CMD;
    s_net_write_cmd(k,cmd_ok);
    sprintf(ins,"[%s:%d] cmd %s (finish)",k->hostname,k->port,cmd_sushr);
    log_msg('I',ins,0);
  }
  else
  {
    decomp_net_us_hr(&tmp_us_hr,ins);
  }
}

/***************************************************************************
*
***************************************************************************/
tus_ll tmp_us;

void set_us_mode(tkon *k,char *ins)
{
  void sucess(void)
  {
    s_net_write_cmd(k,cmd_ok);
    sprintf(ins,"[%s:%d] cmd %s (finish)",k->hostname,k->port,cmd_sus);
    log_msg('I',ins,0);
  }

  if(!strcmp(ins,cmd_ok))
  {
    tus_ll *u;

    u = find_us(tmp_us.user);
    if(u == NULL)
    {
      if(append_us(&tmp_us))
        sucess();
      else
      {
        s_net_write_cmd(k,cmd_er);
        sprintf(ins,"[%s:%d] cmd %s (failed)",k->hostname,k->port,cmd_sus);
        log_error(ins);
      }
    }
    else
    {
      memcpy(u,&tmp_us,TUS_LL_SIZE);
      sucess();
    }
    k->mode = TKON_MODE_CMD;
  }  
  else  
  {
    decomp_net_us_ll(&tmp_us,ins);
  }
}

/***************************************************************************
*
***************************************************************************/
tum_ll tmp_um;

void set_um_mode(tkon *k,char *ins)
{
  if(!strcmp(ins,cmd_ok))
  {
    tum_ll *u;

    u = find_um(tmp_um.user);
    if(u != NULL)
    {
      u->tt_used = tmp_um.tt_used;
      u->dt_used = tmp_um.dt_used;
      u->t_ul = tmp_um.t_ul;
      u->t_dl = tmp_um.t_dl;
      u->d_ul = tmp_um.d_ul;
      u->d_dl = tmp_um.d_dl;
      u->tmp_flags = tmp_um.tmp_flags;
      s_net_write_cmd(k,cmd_ok);
      sprintf(ins,"[%s:%d] cmd %s (finish)",k->hostname,k->port,cmd_sum);
      log_msg('I',ins,0);
    }
    else
    {
      s_net_write_cmd(k,cmd_er);
      sprintf(ins,"[%s:%d] cmd %s (failed)",k->hostname,k->port,cmd_sum);
      log_error(ins);
    }
    k->mode = TKON_MODE_CMD;
  }  
  else  
  {
    decomp_net_um_ll(&tmp_um,ins);
  }
}

/***************************************************************************
*
***************************************************************************/
void service_kons(void)
{
  register int kon_no;

  for(kon_no = 0;kon_no < MAX_KONS;kon_no++)
  {
    register int run;

    for(run = 0;run < 256;run++)
      if(kon[kon_no].s != -1)
      {
        char ins[STR_LEN];

        if(s_net_readln(&(kon[kon_no]),ins,STR_LEN_LN) > 0)
        {
          kon[kon_no].time = timer;

          switch(kon[kon_no].mode)
          {
            case TKON_MODE_CMD:
              cmd_mode(&(kon[kon_no]),ins);
              break;
            case TKON_MODE_S_US_HR:
              set_us_hr_mode(&(kon[kon_no]),ins);
              break;
            case TKON_MODE_S_US:
              set_us_mode(&(kon[kon_no]),ins);
              break;
            case TKON_MODE_S_UM:
              set_um_mode(&(kon[kon_no]),ins);
              break;
          }
        }
        else
        {
          if((timer - kon[kon_no].time) > KON_TIMEOUT)
          {
            s_net_write_cmd(&(kon[kon_no]),cmd_er);
            sprintf(ins,"[%s:%d] connection timeout",
              kon[kon_no].hostname,kon[kon_no].port);
            log_msg('I',ins,1);
            s_close_kon(&(kon[kon_no]));
          }
          break;
        }
      }
  }
}
