/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* message notifier
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <utmp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
#include "log.h"
#include "sm.h"
#include "clobber_user.h"
#include "monitor.h"
#include "net_server.h"

/***************************************************************************
*
***************************************************************************/
int compose_message(char *filename,tmlist *mlist,char **buf)
{
  register int status = 0,m_size,s_size;
  FILE *mfp,*sfp;

  if((mfp = fopen(filename,"r")) != NULL)
  {
    if((sfp = fopen(CLOB_SIGNATURE,"r")) != NULL)
    {
      fseek(mfp,0,SEEK_END);
      m_size = ftell(mfp);
      fseek(mfp,0,SEEK_SET);
      fseek(sfp,0,SEEK_END);
      s_size = ftell(sfp);
      fseek(sfp,0,SEEK_SET);
      if((*buf = NMALLOC(m_size + s_size + 1)) != NULL)
      {
        if(fread(*buf,1,m_size,mfp) == m_size)
          if(fread((*buf) + m_size,1,s_size,sfp) == s_size)
          {
            register int e = 0;

            *((*buf) + m_size + s_size) = 0;
            status = 1;
            for(;;)
            {
              char *p;

              if((mlist + e)->mod == NULL)
                break;
              if((p = strstr(*buf,(mlist + e)->mod)) != NULL)
              {
                register int d,f = strlen((mlist + e)->repl);

                for(d = 0;d < strlen((mlist + e)->mod);d++)
                  if(d < f)
                    p[d] = (mlist + e)->repl[d];
                  else
                    p[d] = ' ';
              }
              e++;
            }
          }
        if(!status) NFREE(*buf);
      }
      fclose(sfp);
    }
    fclose(mfp);
  }
  return status;
}

/***************************************************************************
*
***************************************************************************/
void email_user(char *email_addr,char *msg)
{
  FILE *fp;
  char str[STR_LEN];

  strcpy(str,SENDMAIL_PATH);
  strcat(str," ");
  strcat(str,email_addr);
  if((fp = popen(str,"w")) != NULL)
  {
    fprintf(fp,"From: %s@%s\n",us_hr.root_emailname,us_hr.email_host);
    fprintf(fp,"%s",msg);
    pclose(fp);
  }
}

#define tty_printf(s,d) write(s,d,strlen(d))

void flash_terminal(char *user,char *msg)
{
  struct utmp *utmp_ptr;

  setutent();
  for(;;)
  {
    utmp_ptr = getutent();
    if(utmp_ptr == NULL)
      break;
    if(utmp_ptr->ut_type == USER_PROCESS)
      if(!strncmp(utmp_ptr->ut_user,user,UT_NAMESIZE))
      {
        register int fp;
        char str[STR_LEN];

        strcpy(str,"/dev/");
        strncat(str,utmp_ptr->ut_line,STR_TTY_LEN);
        fp = open(str,O_WRONLY | O_NOCTTY | O_APPEND | O_SYNC);
        if(fp != -1)
        {
          tty_printf(fp,"\a\n");
          tty_printf(fp,"--\n");
          sprintf(str,"From: %s@%s\n",us_hr.root_emailname,us_hr.email_host);
          tty_printf(fp,str);
          tty_printf(fp,msg);
          fsync(fp);
          close(fp);
        }
      }
  }
  endutent();
}

/***************************************************************************
*
***************************************************************************/
void notify_user(tus_ll *us,tum_ll *um,int type)
{
  char str[STR_LEN] = "",*msg = NULL;
  tmlist *mlist = NULL;
  char tmp_str1[STR_GEN_LEN];
  char tmp_str2[STR_GEN_LEN];
  char tmp_str3[STR_GEN_LEN];
  char tmp_str4[STR_GEN_LEN];
  char tmp_str5[STR_GEN_LEN];
  char tmp_str6[STR_GEN_LEN];
  register int r;

  tmlist general_mlist[2] =
  {
    {"%1______________________________",um->user},
    {NULL,NULL}
  };
  tmlist acct_expired_mlist[3] =
  {
    {"%1______________________________",um->user},
    {"%2________",tmp_str1},
    {NULL,NULL}
  };
  tmlist tt_exceeded_mlist[5] =
  {
    {"%1______________________________",um->user},
    {"%2______",tmp_str1},
    {"%3______",tmp_str2},
    {"%4___",tmp_str3},
    {NULL,NULL}
  };
  tmlist dt_exceeded_mlist[6] =
  {
    {"%1______________________________",um->user},
    {"%2______",tmp_str1},
    {"%3______",tmp_str2},
    {"%4______",tmp_str3},
    {"%5___",tmp_str4},
    {NULL,NULL}
  };
  tmlist tnet_exceeded_mlist[9] =
  {
    {"%1______________________________",um->user},
    {"%2______________",tmp_str1},
    {"%3______________",tmp_str2},
    {"%4______________",tmp_str3},
    {"%5______________",tmp_str4},
    {"%6______________",tmp_str5},
    {"%7______________",tmp_str6},
    {NULL,NULL}
  };
  tmlist act_to_mlist[6] =
  {
    {"%1______________________________",um->user},
    {"%2___",tmp_str1},
    {NULL,NULL}
  };

  if(test_mode)
    return;

  r = fork();
  if(r == -1)
    perror("notify_user() fork()");
  else
  {
    if(r == 0) //child
    {
      sprintf(argv_ptr[0],"clobberd -notify %s -code %d",um->user,type);
      switch(type)
      {
        case CLOB_INDEX:
          strcpy(str,CLOB_MESS);
          mlist = general_mlist;
          break;
        case CLOB_MULTIPLE_INDEX:
          strcpy(str,CLOB_MULTPLE_MESS);
          mlist = general_mlist;
          break;
        case ACCT_EXPIRED_INDEX:
          strcpy(str,ACCT_EXPIRED_MESS);
          mlist = acct_expired_mlist;
          compose_date(&(us->expiry_date),tmp_str1);
          break;
        case EXCEEDED_TT_INDEX:
          strcpy(str,EXCEEDED_TT_MESS);
          mlist = tt_exceeded_mlist;
          str_hh_mm(us->tt_lim,tmp_str1);
          str_hh_mm(um->tt_used,tmp_str2);
          str_hh_mm(us->tt_lim - um->tt_used,tmp_str3);
          break;
        case EXCEEDED_DT_INDEX:
          strcpy(str,EXCEEDED_DT_MESS);
          mlist = dt_exceeded_mlist;
          sprintf(tmp_str1,"%d",um->dt_zone_limit);
          str_hh_mm(um->dt_set_limit,tmp_str2);
          str_hh_mm(um->dt_used,tmp_str3);
          str_hh_mm(um->dt_set_limit - um->dt_used,tmp_str4);
          break;
        case EXCEEDED_TNET_INDEX:
          {
            double x;

            strcpy(str,EXCEEDED_TNET_MESS);
            mlist = tnet_exceeded_mlist;

            x = rnd2meg64(um->t_ul);
            sprintf(tmp_str1,"%.2f",x);
            sprintf(tmp_str2,"%d",us->t_ul_lim);
            sprintf(tmp_str3,"%.2f",us->t_ul_lim - x);

            x = rnd2meg64(um->t_dl);
            sprintf(tmp_str4,"%.2f",x);
            sprintf(tmp_str5,"%d",us->t_dl_lim);
            sprintf(tmp_str6,"%.2f",us->t_dl_lim - x);
          }
          break;
        case EXCEEDED_DNET_INDEX:
          {
            double x;

            strcpy(str,EXCEEDED_DNET_MESS);
            mlist = tnet_exceeded_mlist;

            x = rnd2meg64(um->d_ul);
            sprintf(tmp_str1,"%.2f",x);
            sprintf(tmp_str2,"%d",us->d_ul_lim);
            sprintf(tmp_str3,"%.2f",us->d_ul_lim - x);

            x = rnd2meg64(um->d_dl);
            sprintf(tmp_str4,"%.2f",x);
            sprintf(tmp_str5,"%d",us->d_dl_lim);
            sprintf(tmp_str6,"%.2f",us->d_dl_lim - x);
          }
          break;
        case UACT_TIMEOUT_INDEX:
          strcpy(str,UACT_TIMEOUT_MESS);
          mlist = act_to_mlist;
          str_hh_mm(us->act_timeout,tmp_str1);
          break;
      }
      if(compose_message(str,mlist,&msg))
      {
        if(us->flags & TUSF_NOTIFY_VIA_EMAIL)
        {
          if(us->email[0])
            email_user(us->email,msg);
          else
          {
            strcpy(str,um->user);
            strcat(str,"@");
            strcat(str,us_hr.email_host);
            email_user(str,msg);
          }
        }
        else
          flash_terminal(um->user,msg);
      }
      NFREE(msg);
      exit(0);
    }
    else
      inc_child_count();
  }
}
