/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* file io, structure, linked list, btree code etc
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <string.h>
#include "nmalloc.h"
#include "defs.h"
#include "misc.h"
#include "log.h"
#include "monitor.h"
#include "net_server.h"

/***************************************************************************
* fast cache for ppp lock file info in /var/run
* as for 4.9, access this info many many times during a user scan..
***************************************************************************/
int append_nif(char *nif,int pid)
{
  *nif_ll_tail = NMALLOC(sizeof(tnif_ll));
  if(*nif_ll_tail != NULL)
  {
    (*nif_ll_tail)->pid = pid;
    strcpy((*nif_ll_tail)->nif,nif);
    (*nif_ll_tail)->next = NULL;
    nif_ll_tail = &((*nif_ll_tail)->next);
  }
  else
    return 0;
  return 1;
}

tnif_ll *find_nif(int pid)
{
  tnif_ll *s = NULL;

  void f(tnif_ll *p)
  {
    if(p != NULL)
    {
      if(pid == p->pid)
        s = p;
      else
        f(p->next);
    }
  }
  f(nif_ll);
  return s;
}

void free_nif(void)
{
  void f(tnif_ll **entry)
  {
    if(*entry != NULL)
    {
      f(&((*entry)->next));
      NFREE(*entry);
      *entry = NULL;
    }
  }
  f(&nif_ll);
  nif_ll_tail = &nif_ll;
}

/***************************************************************************
* btree struct of US   (fast lookups)
***************************************************************************/
void append_us_btree(tus_ll *entry)
{
  void a(tus_btree **node)
  {
    if(*node != NULL)
    {
      register int r;

      r = strcmp((*node)->entry->user,entry->user);
      if(r < 0)
        a(&((*node)->left));
      if(r > 0)
        a(&((*node)->right));
    }
    else
    {
      *node = NMALLOC(sizeof(tus_btree));
      if(*node != NULL)
      {
        (*node)->entry = entry;
        (*node)->left = NULL;
        (*node)->right = NULL;
      }
    }
  }
  a(&us_btree);
}

void build_us_btree(void)
{
  void b(tus_ll *entry)
  {
    if(entry != NULL)
    {
      append_us_btree(entry);
      b(entry->next);
    }
  }
  b(us_ll);
}

void free_us_btree(tus_btree **node)
{
  if(*node != NULL)
  {
    free_us_btree(&((*node)->left));
    free_us_btree(&((*node)->right));
    NFREE(*node);
    *node = NULL;
  }
}

tus_ll *find_us(char *user)
{
  tus_ll *s = NULL;

  void f(tus_btree *node)
  {
    if(node != NULL)
    {
      if(node->entry != NULL)
      {
        register int r = -1;

        r = strcmp(node->entry->user,user);
        if(r < 0)
          f(node->left);
        else
          if(r > 0)
            f(node->right);
          else
            s = node->entry;
      }
    }
  }
  f(us_btree);
  return s;
}

tus_ll *find_us2(char *user)
{
  tus_ll *s;

  s = find_us(user);
  if(s == NULL)
    if(us_hr.flags & TUSF_HR_DEFAULT_US_ENABLED)
      s = &(us_hr.default_us);
  return s;
}

/***************************************************************************
* manages user settings data 
***************************************************************************/
int append_us(tus_ll *entry) /*Nb/ appends data in sorted order*/
{
  int s = 1;

  void sar(tus_ll **n)
  {
    if(*n != NULL)
    {
      if(strcmp(entry->user,(*n)->user) < 0)
      {
        tus_ll *tmp;

        tmp = NMALLOC(sizeof(tus_ll));
        if(tmp != NULL)
        {
          tus_ll *tmp2;

          memcpy(tmp,entry,TUS_LL_SIZE);
          tmp2 = (*n);
          (*n) = tmp;
          (*n)->next = tmp2;
          append_us_btree(tmp);
        }
        else
          s = 0;
      }
      else
        sar(&((*n)->next));
    }
    else
    {
      *n = NMALLOC(sizeof(tus_ll));
      if((*n) != NULL)
      {
        memcpy((*n),entry,TUS_LL_SIZE);
        (*n)->next = NULL;
        append_us_btree(*n);
      }
      else
        s = 0;
    }
  }
  sar(&us_ll);
  return s;
}

int remove_us(char *user)
{
  int s = 0;

  void r(tus_ll **entry)
  {
    if(*entry != NULL)
    {
      if(!strcmp((*entry)->user,user))
      {
        tus_ll *d;

        d = *entry;
        *entry = (*entry)->next;
        NFREE(d);
        s = 1;
      }
      else
        r(&((*entry)->next));
    }
  }

  free_us_btree(&us_btree);
  r(&us_ll);
  build_us_btree();
  return s;
}

void free_us(void)
{
  void f_ll(tus_ll **entry)
  {
    if(*entry != NULL)
    {
      f_ll(&((*entry)->next));
      NFREE(*entry);
      *entry = NULL;
    }
  }
  f_ll(&us_ll);
  free_us_btree(&us_btree);
}

/***************************************************************************
* manage terminal info data (linked list within um linked list)
***************************************************************************/
int append_tinfo(tum_ll *um_entry,ttinfo_ll *entry)  /*unsorted append*/
{
  int s = 0;

  void f(ttinfo_ll **p)
  {
    if(*p == NULL)
    {
      *p = NMALLOC(sizeof(ttinfo_ll));
      if(*p != NULL)
      {
        memcpy(*p,entry,TTINFO_LL_SIZE);
        (*p)->next = NULL;
        s = 1;
      }
    }
    else
      f(&((*p)->next));
  }
  f(&(um_entry->tinfo_ll));
  return s;
}

/*
 * search by terminal pid
 */
ttinfo_ll *find_tinfo(tum_ll *um_entry,int pid)
{
  ttinfo_ll *s = NULL;

  void f(ttinfo_ll *p)
  {
    if(p != NULL)
    {
      if(p->pid == pid)
        s = p;
      else
        f(p->next);
    }
  }
  f(um_entry->tinfo_ll);
  return s;
}

int remove_tinfo(tum_ll *um_entry,int pid)
{
  int s = 0;

  void r(ttinfo_ll **entry)
  {
    if(*entry != NULL)
    {
      if((*entry)->pid == pid)
      {
        ttinfo_ll *d;
        
        d = *entry;
        *entry = (*entry)->next;
        NFREE(d);
        s = 1;
      }
      else
        r(&((*entry)->next));
    }
  }
  r(&(um_entry->tinfo_ll));
  return s;
}

void free_tinfo(tum_ll *um_entry)
{
  void f(ttinfo_ll **entry)
  {
    if(*entry != NULL)
    {
      f(&((*entry)->next));
      NFREE(*entry);
      *entry = NULL;
    }
  }
  f(&(um_entry->tinfo_ll));
}

/***************************************************************************
* manages user monitor data
***************************************************************************/
int append_um(tum_ll *entry)   /*Nb/ appends data in sorted order*/
{
  int s = 1;
    
  void sar(tum_ll **n)
  {
    if(*n != NULL)
    {
      if(strcmp(entry->user,(*n)->user) < 0)
      {
        tum_ll *tmp;

        tmp = NMALLOC(sizeof(tum_ll));
        if(tmp != NULL)
        {
          tum_ll *tmp2;
       
          memcpy(tmp,entry,TUM_LL_SIZE);
          tmp2 = (*n);
          (*n) = tmp;
          (*n)->next = tmp2;
        }   
        else
          s = 0;
      }
      else
        sar(&((*n)->next));
    }
    else
    {
      *n = NMALLOC(sizeof(tum_ll));
      if((*n) != NULL)
      {
        memcpy((*n),entry,TUM_LL_SIZE);
        (*n)->next = NULL;
      }
      else
        s = 0; 
    }
  }
  sar(&um_ll);
  return s;
}

tum_ll *find_um(char *user)
{
  tum_ll *s = NULL;

  void f(tum_ll *p)
  {
    if(p != NULL)
    {
      if(!strcmp(p->user,user))
        s = p;
      else
        f(p->next);
    }
  }
  f(um_ll);
  return s;
}

int remove_um(char *user)
{
  int s = 0;

  void r(tum_ll **entry)
  {
    if(*entry != NULL)
    {
      if(!strcmp((*entry)->user,user))
      {
        tum_ll *d;

        free_tinfo(*entry);  /*remove tinfo linked list*/
        d = *entry;
        *entry = (*entry)->next;
        NFREE(d);
        s = 1;
      }
      else
        r(&((*entry)->next));
    }
  }
  r(&um_ll);
  return s;
}

void free_um(void)
{
  void f(tum_ll **entry)
  {
    if(*entry != NULL)
    {
      f(&((*entry)->next));
      free_tinfo(*entry);   /*remove tinfo linked list*/
      NFREE(*entry);
      *entry = NULL;
    }
  }
  f(&um_ll);
}

/***************************************************************************
*
***************************************************************************/
int us_hr_locked(void)
{
  register int a;
    
  for(a = 0;a < MAX_KONS;a++)
    if(kon[a].s != -1)
      if(kon[a].flags & TKON_US_HR_LOCKED)
        return 1;
  return 0;
}

int user_locked(char *u)
{
  register int a;

  for(a = 0;a < MAX_KONS;a++)
    if(kon[a].s != -1) 
      if(!strcmp(kon[a].user_lock,u))
        return 1;
  return 0;
}

/*
 * these are all temperary fields that aren't saved to disk
 * they are within clobberd daemon only and must be inited
 */
void init_um(tum_ll *um)
{
  um->dt_zone_limit = -1;
  um->dt_set_limit = 0;
  um->old_ul = 0;
  um->old_dl = 0;
  um->last_active = timer;
  um->tmp_flags = 0;
}

/***************************************************************************
*
***************************************************************************/
int load_data_file(void)
{
  FILE *fp;
  char str[STR_LEN + 1];
  int mode = 0;
  tus_ll tmp_us;
  tum_ll tmp_um;

  if((fp = fopen(DATA_FILE,"r")) == NULL)
  {
    log_error("load_data_file()");
    return 0;
  }
  us_hr_clr(&us_hr);
  us_ll_clr(&tmp_us);
  memset(&tmp_um,0,TUM_LL_SIZE);
  while(!feof(fp))
  {
    if(freadln(fp,str))
    {
      if(str[0] == ';')
      {
        if(!strncmp(str,cmd_ok,strlen(cmd_ok)))
        {
          switch(mode)
          {
            case 2:
              append_us(&tmp_us);
              us_ll_clr(&tmp_us);
              break;   
            case 3:
              append_um(&tmp_um);
              memset(&tmp_um,0,TUM_LL_SIZE);
              break;
          }
          mode = 0;
        }
        else
        {
          if(!strncmp(str,cmd_gushr,strlen(cmd_gushr)))
            mode = 1;
          else if(!strncmp(str,cmd_gus,strlen(cmd_gus)))
            mode = 2;
          else if(!strncmp(str,cmd_gum,strlen(cmd_gum)))
            mode = 3;
        }
      }
      switch(mode)
      {
        case 1:
          decomp_net_us_hr(&us_hr,str);
          break;
        case 2:
          decomp_net_us_ll(&tmp_us,str);
          break;
        case 3:
          decomp_net_um_ll(&tmp_um,str);
          break;
      }
    }
  }
  fclose(fp);
  return 1;
}

/***************************************************************************
*
***************************************************************************/
int save_data_file(void)
{
  FILE *fp;
  char str[STR_BIG] = "";

  void save_us_ll(tus_ll *entry)
  {
    if(entry != NULL)
    {
      fprintf(fp,"\n%s\n",cmd_gus);
      str[0] = 0;
      comp_net_us_ll(entry,str);
      fputs(str,fp);
      fprintf(fp,"%s\n",cmd_ok);
      save_us_ll(entry->next);
    }
  }

  void save_um_ll(tum_ll *entry)
  {
    if(entry != NULL)
    {
      fprintf(fp,"\n%s\n",cmd_gum);
      str[0] = 0;
      comp_net_um_ll(entry,str);
      fputs(str,fp);
      fprintf(fp,"%s\n",cmd_ok);
      save_um_ll(entry->next);
    }
  }

  if((fp = fopen(DATA_FILE,"w")) == NULL)
  {
    log_error("save_data_file()");
    return 0;
  }
  fprintf(fp,"# clobberd " VERSION "\n\n");

  fprintf(fp,"%s\n",cmd_gushr);
  str[0] = 0;
  comp_net_us_hr(&us_hr,str);
  fputs(str,fp);
  fprintf(fp,"%s\n",cmd_ok);

  save_us_ll(us_ll);
  save_um_ll(um_ll);
  fclose(fp);
  return 1;
}
