/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
extern "C" {
#include <curses.h>
#include <string.h>
#include "../clobberd/nmalloc.h"
#include "../clobberd/defs.h"
#include "../clobberd/misc.h"
}
#include "gui.h"
#include "misc.h"
#include "nclient.h"
#include "us.h"

#include "gen_settings_obj.h"

/***************************************************************************
* Edit General Settings
***************************************************************************/
tgen_settings::tgen_settings(void)
{
}

tgen_settings::~tgen_settings(void)
{
}

void tgen_settings::set_mess(char *str,int index)
{
  sprintf(s,"%s",str);
  if(strlen(s) > 40) s[39] = '+';
  s[40] = 0;
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_run_level(int index)
{
  switch(TUSF_HR_G_RUN_LEVEL(us_hr.flags))
  {
    case 0:
      setting_items.ammend_str(index,28,verbose_label);
      break;
    case 1:
      setting_items.ammend_str(index,28,normal_label);
      break;
    case 2:
      setting_items.ammend_str(index,28,none_label);
      break;
  }
}

void tgen_settings::set_sleep_t(int index)
{
  str_mm_ss(us_hr.sleep_time,s);
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_grace_time(int index)
{
  str_hh_mm(us_hr.grace_time,s);
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_grace_volume(int index)
{
  sprintf(s,"%d",us_hr.grace_volume);
  setting_items.ammend_str(index,28,s);
}

void tgen_settings::set_min_dialins(int index)
{
  if(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT)
    setting_items.ammend_str(index,28,exempt_label);
  else
  {
    sprintf(s,"%d",us_hr.min_free_dialins);
    setting_items.ammend_str(index,28,s);
  }
}

void tgen_settings::set_save_time(int index)
{
  str_hh_mm(us_hr.save_time,s);
  setting_items.ammend_str(index,28,s);
} 

void tgen_settings::set_def_us_enabled(int index)
{ 
  if(us_hr.flags & TUSF_HR_DEFAULT_US_ENABLED)
    setting_items.ammend_str(index,28,yes_label);
  else
    setting_items.ammend_str(index,28,no_label);
}

int tgen_settings::create(void)
{
  notify_on(notify_label,pwait_mess);
  if(net_lock_us_hr() != 2)
  {
    notify_off();
    return 0;
  }
  net_get_us_hr();
  tdialog::create(
    "General Settings",
    12,71,3,3,
    COLOR_PAIR(4) | A_BOLD);
  setting_items.init(pos_y + 1,pos_x + 1,10,68,0);

  setting_items.append(
  "Root Email Name           :                                 ");
  setting_items.append(
  "Email Host                :                                         ");
  setting_items.append("Run Level                 :        ");
  setting_items.append("Sleep time of mont (mm:ss):        ");
  setting_items.append("Grace Time (hh:mm)        :        ");
  setting_items.append("Grace Volume (Mb)         : dddddddddd");
  setting_items.append("Master Save Time (hh:mm)  :        ");
  setting_items.append("Min Free Dialins(DT Exmpt):        ");
  setting_items.append("Default US Setting enabled: xxx");
  setting_items.append("Edit Default US Settings  >");

  set_mess(us_hr.root_emailname,0);
  set_mess(us_hr.email_host,1);
  set_run_level(2);
  set_sleep_t(3);
  set_grace_time(4);
  set_grace_volume(5);
  set_save_time(6);
  set_min_dialins(7);
  set_def_us_enabled(8);

  tobj::create(this);
  return 1;
}

void tgen_settings::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  setting_items.redraw();
}

void tgen_settings::run(void)
{
  register int x;

  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        setting_items.cursor_up();
        update();
        break;
      case KEY_DOWN:
        setting_items.cursor_down();
        update();
        break;
      case 'E':
        touched = 1;
        switch(setting_items.get_option())
        {
          case 7:
            us_hr.flags ^= TUSF_HR_MIN_FREE_DIALIN_EXEMPT;
            set_min_dialins(7);
            goto do_redraw;
        }        
        break;
      case 10:
        switch(setting_items.get_option())
        {
          case 0: //root emailname
            touched = 1;
            setting_items.get_string(
              us_hr.root_emailname,28,STR_USER_LEN,STR_USER_LEN);
            set_mess(us_hr.root_emailname,0);
            goto do_redraw;
          case 1: //Email host
            touched = 1;
            setting_items.get_string(us_hr.email_host,28,STR_GEN_LEN,40);
            set_mess(us_hr.email_host,1);
            goto do_redraw;
          case 2: //Run Level
            touched = 1;
            x = TUSF_HR_G_RUN_LEVEL(us_hr.flags);
            x++;
            if(x > 2) x = 0;
            TUSF_HR_S_RUN_LEVEL(us_hr.flags,x);
            set_run_level(2);
do_redraw:
            setting_items.redraw();
            update();
            break;
          case 3: //Sleep time
            touched = 1;
            us_hr.sleep_time = setting_items.get_min_sec(28);
            range_check(&(us_hr.sleep_time),10,99 * 60);
            set_sleep_t(3);
            goto do_redraw;
          case 4: //Grace Time
            touched = 1;
            us_hr.grace_time = setting_items.get_hr_min(28);
            range_check(&(us_hr.grace_time),0,99 * 60 * 60);
            set_grace_time(4);
            goto do_redraw;
          case 5: //Grace Volume
            touched = 1;
            us_hr.grace_volume = setting_items.get_integer(28);
            range_check(&(us_hr.grace_volume),0,0x7fffffff);
            set_grace_volume(5);
            goto do_redraw;
          case 6: //save time
            touched = 1;
            us_hr.save_time = setting_items.get_hr_min(28);
            range_check(&(us_hr.save_time),60,99 * 60 * 60);
            set_save_time(6);
            goto do_redraw;
          case 7: //Min Free Dialins
            if(!(us_hr.flags & TUSF_HR_MIN_FREE_DIALIN_EXEMPT))
            {
              touched = 1;
              us_hr.min_free_dialins = setting_items.get_integer(28);
              range_check(&(us_hr.min_free_dialins),0,0x7fffffff);
              set_min_dialins(7);
              goto do_redraw;
            }
            break;
          case 8:
            touched = 1;
            us_hr.flags ^= TUSF_HR_DEFAULT_US_ENABLED;
            set_def_us_enabled(8);
            goto do_redraw;
          case 9: //Edit Default User Settings
            {
              tedit_def_us edit_def_us;

              edit_def_us.create(
                &(us_hr.default_us),"Default User Settings",4,5);
              edit_def_us.run();
              edit_def_us.dest();
              if(edit_def_us.touched)
                touched = 1;
            }
            break;
        }
        break;
    }
  }
  notify_on(notify_label,pwait_mess);
  if(touched)
    net_put_us_hr();
  net_unlock_us_hr();
}
