/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
extern "C" {
#include <curses.h>
#include "../clobberd/nmalloc.h"
#include "../clobberd/defs.h"
#include "../clobberd/misc.h"
}
#include "gui.h"
#include "misc.h"
#include "nclient.h"
#include "dt.h"
#include "gen_settings.h"
#include "us.h"
#include "um.h"
#include "monitor.h"
#include "kon_stat.h"

#include "main_menu_obj.h"

/***************************************************************************
* Main Menu
***************************************************************************/
tmain_menu::tmain_menu(void)
{
}

tmain_menu::~tmain_menu(void)
{
}

void tmain_menu::create(void)
{
  tdialog::create(
    "Main Menu",
    13,30,2,1,
    COLOR_PAIR(4) | A_BOLD);
  menu_items.init(pos_y + 1,pos_x + 1,11,28,0);
  menu_items.append("Edit General Settings");        //0
  menu_items.append("Edit User Settings");           //1
  menu_items.append("Edit User Monitors");           //2
  menu_items.append("Monitor");                      //3
  menu_items.append("Daily Purge");                  //4
  menu_items.append("Total Purge (and Report)");     //5
  menu_items.append("Print Report Only");            //6
  menu_items.append("Display Connection stats");     //7
  menu_items.append("Force data to be saved");       //8
  menu_items.append("Kill the Master and quit");     //9
  menu_items.append("Quit");                         //10
  tobj::create(this);
}

void tmain_menu::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  menu_items.redraw();
}

void tmain_menu::quit(void)
{
  tquit quit;

  quit.create("Quitting?","Do you wish to Quit (Y/N): ");
  running = quit.run();
  quit.dest();
}

int tmain_menu::warn(void)
{
  tquit quit;
  int s;

  quit.create("Warning!","Are you sure you want to do this? (Y/N): ");
  s = quit.run();
  quit.dest();
  return s;
}

void tmain_menu::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        quit();
        break;
      case KEY_UP:
        menu_items.cursor_up();
        update();
        break;
      case KEY_DOWN:
        menu_items.cursor_down();
        update();
        break;
      case 10:
        switch(menu_items.get_option())
        {
          case 0:
            display_lock();
            {
              tgen_settings gen_settings;

              if(gen_settings.create())
              {
                gen_settings.run();
                gen_settings.dest();
              }
            }
            undisplay_lock();
            break;
          case 1:
            display_lock();
            {
              tedit_us edit_us;

              if(edit_us.create())
              {
                edit_us.run();
                edit_us.dest();
              }
            }
            undisplay_lock();
            break;
          case 2:
            display_lock();
            {
              tedit_um edit_um;

              if(edit_um.create())
              {
                edit_um.run();
                edit_um.dest();
              }
            }
            undisplay_lock();
            break;
          case 3:
            {
              tmon_um mon_um;

              if(mon_um.create())
              {
                mon_um.run();
                mon_um.dest();
              }
            }
            break;
          case 4:
            if(!warn())
            {
              notify_on(notify_label,pwait_mess);
              net_dpurge();
              notify_off();
            }
            break;
          case 5:
            if(!warn())
            {
              notify_on(notify_label,pwait_mess);
              net_tpurge();
              notify_off();
            }
            break;
          case 6:
            notify_on(notify_label,pwait_mess);
            net_treport();
            notify_off();
            break;
          case 7:
            {
              tkon_stat kon_stat;

              if(kon_stat.create())
              {
                kon_stat.run();
                kon_stat.dest();
              }
            }
            break;
          case 8:
            notify_on(notify_label,"Saving..");
            net_put_cmd(cmd_save);
            net_get_ok();
            notify_off();
            break;
          case 9:
            quit();
            if(!running)
            {
              notify_on(notify_label,"B O O O O O M M  ! ! ! !");
              net_put_cmd(cmd_die);
              net_get_ok();
            }
            break;
          case 10:
            quit();
            break;
        }
        break;
    }
  }
}
