/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 96,97,98,1999 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* this is the monitor section
*
****************************************************************************
****************************************************************************
***************************************************************************/
extern "C" {
#include <curses.h>
#include <unistd.h>
#include <string.h>
#include "../clobberd/nmalloc.h"
#include "../clobberd/defs.h"
#include "../clobberd/misc.h"
}
#include "gui.h"
#include "misc.h"
#include "nclient.h"
#include "dt.h"
#include "us.h"
#include "um.h"

#include "monitor_obj.h"

/***************************************************************************
*
***************************************************************************/
void do_count_down(int sleep_time,int *running)
{
  if((*running % 50) == 0)
  {
    char str[10];

    move(LINES - 1,69);
    if(has_colors())
      attrset(COLOR_PAIR(6) | A_BOLD);
    else
      attron(A_REVERSE);
    scroll_clear_line(10);
    move(LINES - 1,69);
    str_mm_ss(sleep_time - (*running / 50),str);
    addstr(str);
    attrset(COLOR_PAIR(5));
    update();
  }
  usleep(10000);
}

int inc_running(int sleep_time,int *running)
{
  if(*running == -1)
    return 0;
  else
    if(*running < (sleep_time * 50))
      (*running)++;
    else
      return 0;
  return 1;
}

/***************************************************************************
* View Monitor Info
***************************************************************************/
tview_terminfo::tview_terminfo(void)
{
  term_list = NULL;
}

tview_terminfo::~tview_terminfo(void)
{
  if(term_list != NULL)
    delete(term_list);
}

int tview_terminfo::get_terminfos(void)
{
  if(term_list != NULL)  //this is here so that list can be reloaded
    delete(term_list);
  term_list = new(tscroll_select);
  if(term_list != NULL)
  {
    char str[STR_LEN_LN + 1];
    ttinfo_ll i;
    int sleep_time,curr_time;

    term_list->init(pos_y + 1,pos_x + 1,length,70,1);
    term_list->append(
"Type        PID   TTY      NetIF    UL (Mb)  DL (Mb)  L. Active  ");

    net_get_stime(&sleep_time,&curr_time);
    sprintf(str,"%s %s",cmd_gtinf,user);
    net_put_cmd(str);
    for(;;)
    {
      if(!net_get_entry(str))
        return 0;
      if(!strcmp(str,cmd_er))
        return 1;
      if(!strcmp(str,cmd_ok))
        return 2;
      if(!strcmp(str,":@"))
      {
        str[0] = 0;
        memset(&i,0,sizeof(ttinfo_ll));
        continue;
      }
      if(!strcmp(str,":#"))
      {
        char str2[STR_GEN_LEN];

        if(i.type > TINFO_TYPE_PPP_UNKNOWN)
          i.type = TINFO_TYPE_UNKNOWN;
        str_hh_mm(curr_time - i.last_active,str2);
        if(i.type == TINFO_TYPE_LOGIN)
          sprintf(str,"%-11s %-5d %-8s %-8s %-8s %-8s %s",
            term_type[i.type],i.pid,i.tty,i.nif,
            na_label,na_label,str2);
        else
          sprintf(str,"%-11s %-5d %-8s %-8s %-8.2f %-8.2f %s",
            term_type[i.type],i.pid,i.tty,i.nif,
            rnd2meg(i.ul),rnd2meg(i.dl),str2);
        term_list->append(str);
        continue;
      }
      decomp_net_tinfo_ll(&i,str);
    }
  }
  else
    return 0;
  return 1;
}

int tview_terminfo::create(char *u)
{
  user = u;
  tdialog::create(
    "View Terminal Info",
    19,70 + 3,3,3,
    COLOR_PAIR(4) | A_BOLD);
  length = 17;
  notify_on(notify_label,pwait_mess);
  if(!get_terminfos())
  {
    notify_off();
    return 0;
  }
  tobj::create(this);
  return 1;
}

void tview_terminfo::redraw(void)
{
  tdialog::redraw();
  attrset(COLOR_PAIR(5));
  term_list->redraw();
}

void tview_terminfo::get_terminfos2(void)
{
  notify_on(notify_label,pwait_mess);
  if(get_terminfos())
  {
    redraw();
    update();
  }
  else
    running = 0;
  notify_off();
}

void tview_terminfo::run(int sleep_time)
{
  running = 0;
  for(;;)
  {
    do_count_down(sleep_time,&running);
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = -1;
        break;
      case ' ':
        get_terminfos2();
        break;
      default:
        if(term_list->list_size)
          switch(ch)
          {
            case KEY_UP:
              term_list->cursor_up();
              update();
              break;
            case KEY_DOWN:
              term_list->cursor_down();
              update();
              break;
          }
    }
    if(!inc_running(sleep_time,&running))
      break;
  }
}

/***************************************************************************
* Peered DT table entry
***************************************************************************/
void tpeer_dt::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        if(hl > 0) hl--;
        draw_table();
        update();
        break;
      case KEY_DOWN:
        if(hl < 7) hl++;
        draw_table();
        update();
        break;
      case KEY_LEFT:
        if(sl > 0) sl--;
        draw_table();
        update();
        break;
      case KEY_RIGHT:
        if(sl < 4) sl++;
        draw_table();
        update();
        break;
    }
  }
}

/***************************************************************************
* Peered US entry
***************************************************************************/
void tpeer_us::open_dt(void)
{
  if(!(curr_us->flags & TUSF_EXPT_DAILY_TIME))
  {
    tpeer_dt peer_dt;

    peer_dt.create(curr_us->dt_lim);
    peer_dt.run();
    peer_dt.dest();
  }
}

void tpeer_us::run(void)
{
  while(running)
  {
    ch = getch();
    switch(ch)
    {
      case 'Q':
        running = 0;
        break;
      case KEY_UP:
        us_select.cursor_up();
        update();
        break;
      case KEY_DOWN:
        us_select.cursor_down();
        update();
        break;
      case 10:
        if(us_select.get_option() == 2)
          open_dt();
        break;
    }
  }
}

/***************************************************************************
* Monitor entry
***************************************************************************/
void tmon_um_entry::create(tum_ll *ump,char *hr,int y,int x)
{
  curr_um = ump;
  tdialog::create(hr,14,53,y,x,COLOR_PAIR(4) | A_BOLD);
  um_select.init(pos_y + 1,pos_x + 1,12,51,0);
  create_sub();
  um_select.append("Peer at US record        >");
  um_select.append("View Terminal Info       >");
  tobj::create(this);
}

void tmon_um_entry::load_peer_us(tus_ll *entry)
{
  tpeer_us peer_us;

  notify_off();
  peer_us.create(entry,"User Settings",4,16);
  peer_us.run();
  peer_us.dest();
}

void tmon_um_entry::select_peer_us(void)
{
  nodelay(stdscr,0);
  notify_on(notify_label,pwait_mess);
  if(net_get_us(curr_um->user,1) == 2)
    load_peer_us(&us);
  else
    notify(notify_label,"Couldn't find US record");
  nodelay(stdscr,1);
}

void tmon_um_entry::select_terminfo(int sleep_time)
{
  int r;

  do
  {
    tview_terminfo view_terminfo;

    view_terminfo.create(curr_um->user);
    view_terminfo.run(sleep_time);
    r = view_terminfo.running;
    view_terminfo.dest();
  } while(r != -1);
}

void tmon_um_entry::run(int sleep_time)
{
  nodelay(stdscr,1);
  running = 0;
  for(;;)
  {
    do_count_down(sleep_time,&running);
    switch(getch())
    {
      case 'Q':
        running = -1;
        break;
      case KEY_UP:
        um_select.cursor_up();
        update();
        break;
      case KEY_DOWN:
        um_select.cursor_down();
        update();
        break;
      case 10:
        switch(um_select.get_option())
        {
          case 10:
            select_peer_us();
            break;
          case 11:
            select_terminfo(sleep_time);
            break;
        }
        break;
    }
    if(!inc_running(sleep_time,&running))
      break;
  }
  nodelay(stdscr,0);
  if(has_colors())
    attrset(COLOR_PAIR(1) | A_BOLD);
  else
    attron(A_REVERSE);
  move(LINES - 1,69);
  scroll_clear_line(10);
}

/***************************************************************************
* Monitor
***************************************************************************/
void tmon_um::select(void)
{
  char *user;
  int sleep_time,curr_time,running = 1;

  user_list->get_str(user_list->get_option(),0,&user);
  do
  {
    notify_on(notify_label,pwait_mess);
    net_get_stime(&sleep_time,&curr_time);
    if(net_get_um(user) == 2)
    {
      tmon_um_entry mon_um_entry;

      um.last_active = curr_time - um.last_active;
      mon_um_entry.monitor_mode = 1;
      mon_um_entry.create(&um,"User Monitor",3,15);
      mon_um_entry.run(sleep_time);
      running = mon_um_entry.running;
      mon_um_entry.dest();
    }
    else
    {
      notify(error_label,"Error loading UM");
      break;
    }
  }
  while(running != -1);
}

void tmon_um::add_user(void)
{
}

void tmon_um::delete_user(void)
{
}
