/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
extern "C" {
#include <curses.h>
#include <string.h>
#include "../clobberd/nmalloc.h"
#include "../clobberd/defs.h"
#include "../clobberd/misc.h"
}
#include "gui.h"
#include "misc.h"

/***************************************************************************
*
***************************************************************************/
int net_get_entry(char *str)
{
  register int s;

  if(client_fp == NULL)
    return 0;
  if(fgets(str,STR_LEN_LN,client_fp) == NULL)
  {
    notify(error_label,nerr_mess);
    fclose(client_fp);
    client_fp = NULL;
    return 0;
  }
  s = strlen(str);
  if(s > 0)
    if(str[s - 1] == '\n')
      str[s - 1] = 0;
  return 1;
}

void net_put_entry(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,str);
  fflush(client_fp);
}

void net_put_cmd(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,"%s\n",str);
  fflush(client_fp);
}

/***************************************************************************
*
***************************************************************************/
int net_get_ok(void)
{
  if(client_fp == NULL)
    return 0;
  for(;;)
  {       
    char str[STR_LEN_LN + 1];

    if(net_get_entry(str) == 0)
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
  } 
  return 2;
}

/***************************************************************************
*
***************************************************************************/
int net_lock_us_hr(void)
{
  register int s;

  if(client_fp == NULL)
    return 0;
  net_put_cmd(cmd_lushr);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,
      "Daemon is locked by another remote maintanence session."
      " Hit any key.");
  return s;
}

int net_unlock_us_hr(void)
{
  if(client_fp == NULL)
    return 0;
  net_put_cmd(cmd_uushr);
  if(net_get_ok() != 2)
    return 0;
  return 1;
}

int net_get_us_hr(void)
{
  if(client_fp == NULL)
    return 0;
  memset(&us_hr,0,sizeof(tus_hr));
  net_put_cmd(cmd_gushr);
  for(;;)
  {
    char str[STR_LEN_LN];

    if(!net_get_entry(str))
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
    decomp_net_us_hr(&us_hr,str);
  }
  return 0;
}

int net_put_us_hr(void)
{
  char str[STR_BIG] = "";

  if(client_fp == NULL)
    return 0;
  comp_net_us_hr(&us_hr,str);
  net_put_cmd(cmd_sushr);
  net_put_entry(str);
  net_put_cmd(cmd_ok);
  return net_get_ok();
}

/***************************************************************************
*
***************************************************************************/
int net_lock_user(char *user)
{
  register int s;
  char str[STR_LEN];

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_luser,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,
      "User is locked by another remote maintanence session. Hit any key.");
  return s;
}

int net_unlock_user(void)
{
  if(client_fp == NULL)
    return 0;
  net_put_cmd(cmd_uuser);
  if(net_get_ok() != 2)
    return 0;
  return 1;
}

/***************************************************************************
* type:
* 0 = find_us()
* 0 = find_us2()
***************************************************************************/
int net_get_us(char *user,int type)
{
  char str[STR_LEN];

  if(client_fp == NULL)
    return 0;
  memset(&us,0,TUS_LL_SIZE);
  sprintf(str,"%s %s",type ? cmd_gus2 : cmd_gus,user);
  net_put_cmd(str);
  for(;;)
  {
    char str[STR_LEN_LN];

    if(!net_get_entry(str))
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
    decomp_net_us_ll(&us,str);
  }
  return 0;
}

int net_put_us(void)
{
  char str[STR_BIG] = "";
  register int s;

  if(client_fp == NULL)
    return 0;
  comp_net_us_ll(&us,str);
  net_put_cmd(cmd_sus);
  net_put_entry(str);
  net_put_cmd(cmd_ok);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,"Error saving User Setting. Hit any key.");
  return s;
}

int net_new_us(char *user)
{
  char str[STR_LEN]; 
  register int s;

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_nus,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,
      "Username MUST be unique. Didn't add this entry.. Hit any key.");
  return s;
}

int net_delete_us(char *user)
{
  register int s;
  char str[STR_LEN]; 

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_dus,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,user_nf_mess);
  return s;
}

/***************************************************************************
*
***************************************************************************/
int net_get_um(char *user)
{
  char str[STR_LEN];

  if(client_fp == NULL)
    return 0;
  memset(&um,0,TUM_LL_SIZE);
  sprintf(str,"%s %s",cmd_gum,user);
  net_put_cmd(str);
  for(;;)
  {
    char str[STR_LEN_LN];

    if(!net_get_entry(str))
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
    decomp_net_um_ll(&um,str);
  }
  return 0;
}

int net_put_um(void)
{
  char str[STR_BIG] = "";
  register int s;

  if(client_fp == NULL)
    return 0;
  comp_net_um_ll(&um,str);
  net_put_cmd(cmd_sum);
  net_put_entry(str);
  net_put_cmd(cmd_ok);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,user_nf_mess);
  return s;
}

int net_delete_um(char *user)
{
  register int s;
  char str[STR_LEN]; 

  if(client_fp == NULL)
    return 0;
  sprintf(str,"%s %s",cmd_dum,user);
  net_put_cmd(str);
  s = net_get_ok();
  if(s == 1)
    notify(notify_label,user_nf_mess);
  return s;
}

/***************************************************************************
*
***************************************************************************/
void net_get_stime(int *s,int *t)
{
  char str[STR_LEN]; 

  *s = 60;
  if(client_fp != NULL)
  {
    net_put_cmd(cmd_stime);
    if(net_get_entry(str))
    {
      if(str[0] == ':')
        sscanf(str + 1,"%x %x",s,t);
      net_get_ok();
    }
  }
}

void net_tpurge(void)
{
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_tpurg);
  net_get_ok();
}

void net_dpurge(void)
{
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_dpurg);
  net_get_ok();
}

void net_treport(void)
{
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_trept);
  net_get_ok();
}
