/***************************************************************************
****************************************************************************
****************************************************************************
*
* Clobberd - By Jason Nunn
* Copyright (C) 1996,1997,1998 Jason Nunn
*
* Clobberd comes under the GNU General Public License. Please read the
* COPYING notice in this distribution.
*
* ================================================================
* clobbertool
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include <utmp.h>
#include <sys/time.h>
#include <sys/types.h>
#include "../clobberd/defs.h"
#include "../clobberd/misc.h"

#define FUNC_USERSTATS      1
#define FUNC_USERSTATS_ALL  2
#define FUNC_DAILY_PURGE    3
#define FUNC_TOTAL_PURGE    4

int  client_s;
FILE *client_fp = NULL;

typedef struct _tuser_ll
{
  char              user[STR_USER_LEN + 1];
  struct _tuser_ll *next;
} tuser_ll;

/***************************************************************************
*
***************************************************************************/
int net_get_entry(char *str)
{
  register int s;

  if(client_fp == NULL)
    return 0;
  if(fgets(str,STR_LEN_LN,client_fp) == NULL)
  {
    printf("Network Error, aborting program.\n");
    fclose(client_fp);
    client_fp = NULL;
    return 0;
  }
  s = strlen(str);
  if(s > 0)
    if(str[s - 1] == '\n')
      str[s - 1] = 0;
  return 1;
}

void net_put_entry(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,str);
  fflush(client_fp);
}

void net_put_cmd(char *str)
{
  if(client_fp == NULL)
    return;
  fprintf(client_fp,"%s\n",str);
  fflush(client_fp);
}

int net_get_ok(void)
{
  if(client_fp == NULL)
    return 0;
  for(;;)
  {       
    char str[STR_LEN_LN + 1];

    if(net_get_entry(str) == 0)
      return 0;
    if(!strcmp(str,cmd_er))
      return 1;
    if(!strcmp(str,cmd_ok))
      return 2;
  } 
  return 2;
}

/***************************************************************************
*
***************************************************************************/
void total_purge(void)
{
  printf("Total Purge..\n");
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_tpurg);
  net_get_ok();
  net_put_cmd(cmd_quit);
}

void daily_purge(void)
{
  printf("Daily Purge..\n");
  if(client_fp == NULL)
    return;
  net_put_cmd(cmd_dpurg);
  net_get_ok();
  net_put_cmd(cmd_quit);
}

/***************************************************************************
*
***************************************************************************/
void user_stat(char *user)
{
  printf("\n");
  if(client_fp != NULL)
  {
    char str[STR_LEN_LN];

    sprintf(str,"%s %s",cmd_ustat,user);
    net_put_cmd(str);
    for(;;)
    {
      if(!net_get_entry(str))
        break;
      if(!strcmp(str,cmd_er))
        break;
      if(!strcmp(str,cmd_ok))
        break;
      printf("%s\n",str + 1);
    }
  }
  printf("\n");
}

void get_all_users(void)
{
  tuser_ll *user_ll = NULL,*p;
  char str[STR_LEN_LN + 1];

  void get_user_ll(tuser_ll **entry)
  {
    if(!net_get_entry(str))
      return;
    if(!strcmp(str,cmd_er))
      return;
    if(!strcmp(str,cmd_ok))
      return;
    if(str[0] == ':')
    {
      *entry = malloc(sizeof(tuser_ll));
      if(*entry != NULL)
      {
        strcpy((*entry)->user,str + 1);
        (*entry)->next = NULL;
      }
    }
    get_user_ll(&((*entry)->next));
  }

  void free_user_ll(tuser_ll **entry)
  {
    if(*entry != NULL)
    {
      free_user_ll(&((*entry)->next));
      free(*entry);
      *entry = NULL;
    }
  }

  net_put_cmd(cmd_gumu);
  get_user_ll(&user_ll);
  p = user_ll;
  while(p != NULL)
  {
    user_stat(p->user);
    p = p->next;
  }
  free_user_ll(&user_ll);
}

/***************************************************************************
*
***************************************************************************/
void print_help(void)
{
  printf(
    DISCLAIMER
    "Usage: clobbertool <options>\n"
    " -h          Display this help info\n"
    " -a <host>   Hostname to connect to (default: localhost)\n"
    " -p <passwd> Password  (needed for -d and -t options)\n"
    " -s <user>   Display User stats\n"
    " -A          Display All User stats\n"
    " -d          Purge Daily Settings\n"
    " -t          Purge Total Settings and Print Report\n"
    "\n");
}

int main(int argc,char *argv[])
{
  register int x,func = 0;
  char *host = "localhost",*user = "";

  printf(WELCOME);
  if(argc < 2)
  {
    print_help();
    return -1;
  }
  boot_st.passwd[0] = 0;
  for(x = 1;x < argc;x++)
  {
    if(argv[x][0] == '-')
    {
      switch(argv[x][1])
      {
        case 'h':
          print_help();
          return 0;
        case 'a':
          if((x + 1) < argc)
            host = argv[x + 1];
          break;
        case 'p':
          if((x + 1) < argc)
          {  //blott out pasword so noone an see it in process table
            strncpy(boot_st.passwd,argv[x + 1],STR_GEN_LEN);
            memset(argv[x + 1],'*',strlen(boot_st.passwd));
          }
          break;
        case 's':
          func = FUNC_USERSTATS;
          if((x + 1) < argc)
            user = argv[x + 1];
          break;
        case 'A':
          func = FUNC_USERSTATS_ALL;
          break;
        case 'd':
          func = FUNC_DAILY_PURGE;
          break;
        case 't':
          func = FUNC_TOTAL_PURGE;
          break;
      }
    }
  }

  if(strlen(boot_st.passwd) > STR_GEN_LEN)
  {
    printf("password too long.\n");
    return -1;
  }
  if(!func)
  {
    print_help();
    return -1;
  }
  if(func > FUNC_USERSTATS)
  {
    struct passwd *pw;

    pw = getpwuid(getuid());
    if(pw != NULL)
      if(strcmp(pw->pw_name,"root"))
      {
        printf("Must be root to run clobberd. Aborting process.\n");
        return -1;
      }
  }
  else
    if(strlen(user) > STR_USER_LEN)
    {
      printf("Username too long.\n");
      return -1;
    }
  client_init();
  if(boot_st.passwd[0] ? 1 : read_boot_conf())
  {
    client_s = client_bind_socket(host,PORT);
    if(client_s)
    {
      printf("Connected to [%s]\n",host);
      client_fp = fdopen(client_s,"a+");
      if(client_fp != NULL)
      {
        if(net_get_ok() == 2)
        {
          switch(func)
          {
            case FUNC_USERSTATS:
              user_stat(user);
              break;
            default:
            {
              char str[STR_LEN];
              register int s;

              sprintf(str,"%s %s",cmd_pass,boot_st.passwd);
              net_put_cmd(str);
              s = net_get_ok();
              if(s == 1)
                printf("Password incorrect. Aborting program.\n");
              else if(s == 2)
              {
                switch(func)
                {
                  case FUNC_USERSTATS_ALL:
                    get_all_users();
                    break;
                  case FUNC_DAILY_PURGE:
                    daily_purge();
                    break;
                  case FUNC_TOTAL_PURGE:
                    total_purge();
                    break;
                }
                printf("Done.\n");
              }
              break;
            }
          }
        }
        if(client_fp != NULL)
          fclose(client_fp);
      }
      close_socket(client_s);
    }
    else
      printf("Daemon is down, aborting program.\n");
  }
  else
    printf("Must specify password. aborting program.\n");
  return 0;
}
